"use strict";

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAll = getAll;
exports.getAttrValuefromKey = getAttrValuefromKey;
exports.getValueFromAttribute = getValueFromAttribute;
exports.sanitizeSourceAttribute = sanitizeSourceAttribute;

var _ = require("../../");

var sourceAttr = _interopRequireWildcard(require("../../cms/editor/handlebars/sourceAttr"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
* Get All attributes from an Abe tag
* @return {Object} parsed attributes
*/
function getAll(str, json) {
  str = _.abeExtend.hooks.instance.trigger('beforeAbeAttributes', str, json); //This regex analyzes all attributes of an Abe tag

  var re = /\b([a-z][a-z0-9\-]*)\s*=\s*("([^"]+)"|'([^']+)'|(\S+))/gi;
  var attrs = {
    autocomplete: null,
    desc: '',
    hint: '',
    display: null,
    editable: true,
    key: '',
    'max-length': null,
    'min-length': 0,
    order: 0,
    prefill: false,
    'prefill-quantity': null,
    reload: false,
    required: false,
    source: null,
    tab: 'default',
    type: '',
    value: null,
    file: '',
    visible: true
  };

  for (var match; match = re.exec(str);) {
    attrs[match[1]] = match[3] || match[4] || match[5];
  }

  attrs.sourceString = attrs.source;
  attrs.source = attrs.source ? json != null && json['abe_source'] != null ? json['abe_source'][attrs.key] : null : null;
  attrs.editable = attrs.editable && attrs.editable !== 'false' ? true : false;
  attrs = _.abeExtend.hooks.instance.trigger('afterAbeAttributes', attrs, str, json);
  return attrs;
}

function sanitizeSourceAttribute(obj, jsonPage) {
  if (obj.sourceString != null && obj.sourceString.indexOf('{{') > -1) {
    var matches = obj.sourceString.match(/({{[a-zA-Z._]+}})/g);

    if (matches !== null) {
      Array.prototype.forEach.call(matches, function (match) {
        var val = match.replace('{{', '');
        val = val.replace('}}', '');

        try {
          val = eval('jsonPage.' + val);
          obj.sourceString = obj.sourceString.replace(match, val);
        } catch (e) {
          val = '';
        }
      });
    }
  }

  return obj;
}

function getAttrValuefromKey(template, key, attr, json) {
  var value = null;

  var abeTags = _.cmsData.regex.getTagAbeWithKey(template, key);

  if (abeTags.length > 0) {
    var attr = _.cmsData.regex.getAttr(abeTags[0], attr);

    if (attr !== '') {
      value = _.cmsData.attributes.getValueFromAttribute(attr, json);
    }
  }

  return value;
}
/**
 * This function will take the value of an Abe attribute and analyze its content.
 * If it contains a var {{variable.prop.value}}, it will extract its content with these rules
 * element.[0].value
 * element[0].value
 * element.0.value
 * will all be transformed into element[0].value and evaluated with the json
 * If it contains a [] ie. {{variable[].value}} or {{variable.[].value}}
 * It will return the array of values
 * @param  {String} value
 * @return {String}
 */


function getValueFromAttribute(value, json) {
  var result = value;

  if (value.indexOf('{{') > -1) {
    var keys = sourceAttr.getKeys(value);
    var isAr = false;
    Array.prototype.forEach.call(keys, function (key) {
      var toEval = "".concat(key.replace(/(\[|\.|\])/g, '\\$1'));
      var properties = key.split('.');
      Array.prototype.forEach.call(properties, function (prop, index) {
        if (prop.indexOf('[]') > 0) {
          isAr = true;
        } else if (prop.indexOf('[]') == 0 && index > 0) {
          properties[index - 1] += prop;
          properties.splice(index, 1);
          isAr = true;
        } else if (prop.indexOf('[') == 0 && index > 0) {
          properties[index - 1] += prop;
          properties.splice(index, 1);
        } else if (/^\d+$/.test(prop) && index > 0) {
          properties[index - 1] += '[' + prop + ']';
          properties.splice(index, 1);
        }
      });
      key = properties.join('.');

      try {
        if (isAr) {
          result = [];
          var properties = key.split('[]');
          var jsonAr = eval("json.".concat(properties[0]));
          Array.prototype.forEach.call(jsonAr, function (prop, index) {
            var resTemp = value;
            result.push(resTemp.replace(new RegExp("{{".concat(toEval, "}}"), 'g'), eval("prop".concat(properties[1]))));
          });
        } else {
          result = result.replace(new RegExp("{{".concat(toEval, "}}"), 'g'), eval("json.".concat(key)));
        }
      } catch (e) {}
    });
  }

  return result;
}