"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.get = get;
exports.getAbeMeta = getAbeMeta;
exports.getAllWithKeys = getAllWithKeys;
exports.getFileObject = getFileObject;
exports.getFilesByType = getFilesByType;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _fs = _interopRequireDefault(require("fs"));

var _ = require("../../");

function getAbeMeta(fileObject, json) {
  if (json.name != null) {
    fileObject.name = json.name;
  }

  if (json.abe_meta != null) {
    var date = null;

    if (json.abe_meta.updatedDate != null) {
      fileObject.date = json.abe_meta.updatedDate;
      date = json.abe_meta.updatedDate;
    } else if (json.abe_meta.latest != null && json.abe_meta.latest.date != null) {
      fileObject.date = json.abe_meta.latest.date;
      date = json.abe_meta.latest.date;
    } else if (json.abe_meta.date !== null) {
      date = json.abe_meta.date;
    }

    fileObject.abe_meta = {
      date: date,
      link: json.abe_meta.link != null ? json.abe_meta.link : null,
      template: json.abe_meta.template != null ? json.abe_meta.template : null,
      status: json.abe_meta.status != null ? json.abe_meta.status : null
    };
  }

  return fileObject;
}

function getAllWithKeys(_x) {
  return _getAllWithKeys.apply(this, arguments);
}

function _getAllWithKeys() {
  _getAllWithKeys = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(withKeys) {
    var extension, files, filesArr;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            extension = '.json';
            _context2.next = 3;
            return _.coreUtils.file.getFiles(_.Manager.instance.pathData, true, extension);

          case 3:
            files = _context2.sent;
            filesArr = [];
            _context2.next = 7;
            return Promise.all(files.map( /*#__PURE__*/function () {
              var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(pathFile) {
                var json, fileObject;
                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _context.next = 2;
                        return _.cmsData.revision.getDoc(pathFile);

                      case 2:
                        json = _context.sent;
                        fileObject = _.cmsData.file.getFileObject(pathFile, json);
                        fileObject = _.cmsData.file.getAbeMeta(fileObject, json);
                        Array.prototype.forEach.call(withKeys, function (key) {
                          fileObject[key] = json[key];
                        });
                        filesArr.push(fileObject);

                      case 7:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              return function (_x3) {
                return _ref.apply(this, arguments);
              };
            }()));

          case 7:
            return _context2.abrupt("return", filesArr);

          case 8:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _getAllWithKeys.apply(this, arguments);
}

function get(_x2) {
  return _get.apply(this, arguments);
}

function _get() {
  _get = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(pathJson) {
    var json, stat;
    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            json = {};
            pathJson = _.abeExtend.hooks.instance.trigger('beforeGetJson', pathJson);
            _context3.prev = 2;
            _context3.next = 5;
            return _fs["default"].statSync(pathJson);

          case 5:
            stat = _context3.sent;

            if (stat) {
              json = _fs["default"].readFileSync(pathJson);
              json = JSON.parse(json);
            }

            _context3.next = 12;
            break;

          case 9:
            _context3.prev = 9;
            _context3.t0 = _context3["catch"](2);
            //console.log('error get', e)
            json = {};

          case 12:
            json = _.abeExtend.hooks.instance.trigger('afterGetJson', json);
            return _context3.abrupt("return", json);

          case 14:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, null, [[2, 9]]);
  }));
  return _get.apply(this, arguments);
}

function getFilesByType(pathFile) {
  var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var extension = '.' + _.config.files.templates.extension;
  var result = [];

  try {
    var directory = _fs["default"].lstatSync(pathFile);

    if (!directory.isDirectory()) {
      _mkdirp["default"].sync(pathFile);
    }
  } catch (e) {
    _mkdirp["default"].sync(pathFile);
  }

  var files = _.coreUtils.file.getFilesSync(pathFile, true, extension);

  Array.prototype.forEach.call(files, function (file) {
    var val = _.cmsData.fileAttr.get(file).s;

    if (type === null || val === type) result.push(file);
  });
  return result;
}

function getFileObject(relativePath, json) {
  var pathFile = _.cmsData.utils.getRevisionPath(relativePath);

  var name = _path["default"].basename(pathFile);

  name = _.cmsData.fileAttr["delete"](name);
  var fileObject = {
    name: name,
    date: _.cmsData.fileAttr.getDate(pathFile),
    path: pathFile
  };
  return fileObject;
}