"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.exist = exist;
exports.filePathInfos = filePathInfos;
exports.getAllWithKeys = getAllWithKeys;
exports.getDoc = getDoc;
exports.getDocumentRevision = getDocumentRevision;
exports.getFileObject = getFileObject;
exports.getFilesMerged = getFilesMerged;
exports.getStatusAndDateToFileName = getStatusAndDateToFileName;
exports.getVersions = getVersions;
exports.mergeRevisions = mergeRevisions;
exports.removeStatusAndDateFromFileName = removeStatusAndDateFromFileName;
exports.sortRevisions = sortRevisions;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _path = _interopRequireDefault(require("path"));

var _lodash = _interopRequireDefault(require("lodash"));

var _2 = require("../../");

/**
 * Create and array of doc file path containing the versions of this doc
 * @param  {String} path of a doc
 * @return {Array} the versions of the doc
 */
function getVersions(docPath) {
  var result = [];

  var files = _2.Manager.instance.getList();

  var jsonPath = docPath.replace("." + _2.config.files.templates.extension, ".json");

  if (files != null) {
    Array.prototype.forEach.call(files, function (file) {
      if (file.path.startsWith(jsonPath)) {
        result = file.revisions;
      }
    });
  }

  return result;
}
/**
 * Return the revision from document html file path
 * if the docPath contains abe revision [status]-[date] will try to return this revision
 * else it will return the latest revision
 * or null
 *
 * @param  {String} html path
 * @return {Object} file revision | null
 */


function getDocumentRevision(docPath) {
  var result = null;
  var documentPath = docPath;
  var latest = true;

  if (_2.cmsData.fileAttr.test(documentPath)) {
    latest = false;
    documentPath = _2.cmsData.fileAttr["delete"](documentPath);
  }

  var revisions = getVersions(documentPath);

  if (latest && revisions.length >= 0) {
    result = revisions[0];
  }

  return result;
}

function getStatusAndDateToFileName(date) {
  var res = date.substring(0, 4) + "-" + date.substring(4, 6) + "-" + date.substring(6, 11) + ":" + date.substring(11, 13) + ":" + date.substring(13, 15) + "." + date.substring(15, 19);
  return res;
}

function removeStatusAndDateFromFileName(date) {
  return date.replace(/[-:\.]/g, "");
}

function filePathInfos(pathFolder) {
  var pathArr = pathFolder.split("/");
  var name = pathArr[pathArr.length - 1];

  var rootArr = _2.config.root.split("/");

  var website = rootArr[pathArr.length - 1];
  return {
    name: name,
    path: pathFolder,
    website: website,
    type: "folder"
  };
}

function mergeRevisions(files) {
  var merged = {};
  Array.prototype.forEach.call(files, function (file) {
    var docRelativePath = _2.cmsData.utils.getDocRelativePath(file.path);

    if (merged[docRelativePath] == null) {
      merged[docRelativePath] = {
        path: _2.cmsData.utils.getDocRelativePath(file.path),
        revisions: []
      };
    }

    merged[docRelativePath].revisions.push(JSON.parse(JSON.stringify(file)));
  });
  return merged;
}

function sortRevisions(merged) {
  var sortedArray = [];
  Array.prototype.forEach.call(Object.keys(merged), function (key) {
    var revisions = merged[key].revisions;
    revisions.sort(_2.coreUtils.sort.predicatBy("date", -1));
    merged[key].revisions = revisions;

    if (revisions[0] != null) {
      merged[key].name = revisions[0].name;
      merged[key].date = revisions[0].date;
      merged[key].abe_meta = revisions[0].abe_meta;
    }

    var newStatuses = [];
    Array.prototype.forEach.call(revisions, function (revision) {
      var revisionStatus = _lodash["default"].get(revision, "abe_meta.status");

      if (revisionStatus && revisionStatus != "") {
        if (!(revisionStatus in newStatuses)) {
          if (revisionStatus === "publish") {
            merged[key][revisionStatus] = revision;
          } else {
            merged[key][revisionStatus] = {};
          }

          merged[key][revisionStatus].path = revision.path;
          merged[key][revisionStatus].date = revision.date;
          merged[key][revisionStatus].link = revision.abe_meta.link;
          newStatuses[revisionStatus] = true;
        }
      }
    });
    sortedArray.push(merged[key]);
  });
  return sortedArray;
}

function getFilesMerged(files) {
  var revisions = _2.cmsData.revision.mergeRevisions(files);

  return _2.cmsData.revision.sortRevisions(revisions);
}

function getDoc(_x) {
  return _getDoc.apply(this, arguments);
}

function _getDoc() {
  _getDoc = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(relativePath) {
    var json;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            json = {};

            if (!(_2.config.database.type == "file")) {
              _context.next = 8;
              break;
            }

            relativePath = _2.cmsData.utils.getRevisionPath(relativePath);
            _context.next = 5;
            return _2.cmsData.file.get(relativePath);

          case 5:
            json = _context.sent;
            _context.next = 12;
            break;

          case 8:
            if (!(_2.config.database.type == "mongo")) {
              _context.next = 12;
              break;
            }

            _context.next = 11;
            return _2.mongo.getDoc(relativePath)["catch"](function (err) {
              console.log(err);
            });

          case 11:
            json = _context.sent;

          case 12:
            return _context.abrupt("return", json);

          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _getDoc.apply(this, arguments);
}

function getAllWithKeys(_x2) {
  return _getAllWithKeys.apply(this, arguments);
}

function _getAllWithKeys() {
  _getAllWithKeys = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(withKeys) {
    var filesArr, merged;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (!(_2.config.database.type == "file")) {
              _context2.next = 6;
              break;
            }

            _context2.next = 3;
            return _2.cmsData.file.getAllWithKeys(withKeys);

          case 3:
            filesArr = _context2.sent;
            _context2.next = 10;
            break;

          case 6:
            if (!(_2.config.database.type == "mongo")) {
              _context2.next = 10;
              break;
            }

            _context2.next = 9;
            return _2.mongo.getAllWithKeys(withKeys);

          case 9:
            filesArr = _context2.sent;

          case 10:
            merged = _2.cmsData.revision.getFilesMerged(filesArr);

            _2.abeExtend.hooks.instance.trigger("afterGetAllFiles", merged);

            return _context2.abrupt("return", merged);

          case 13:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _getAllWithKeys.apply(this, arguments);
}

function exist(_x3) {
  return _exist.apply(this, arguments);
}

function _exist() {
  _exist = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(jsonPath) {
    var exists;
    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            jsonPath = _2.cmsData.utils.getRevisionRelativePath(jsonPath);
            exists = false;

            if (!(_2.config.database.type == "file")) {
              _context3.next = 6;
              break;
            }

            exists = _2.coreUtils.file.exist(_path["default"].join(_2.Manager.instance.pathData, jsonPath));
            _context3.next = 10;
            break;

          case 6:
            if (!(_2.config.database.type == "mongo")) {
              _context3.next = 10;
              break;
            }

            _context3.next = 9;
            return _2.mongo.exist(jsonPath);

          case 9:
            exists = _context3.sent;

          case 10:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _exist.apply(this, arguments);
}

function getFileObject(relativePath, json) {
  var fileObject = {};

  if (_2.config.database.type == "file") {
    fileObject = _2.cmsData.file.getFileObject(relativePath, json);
  } else if (_2.config.database.type == "mongo") {
    fileObject = _2.mongo.getFileObject(json);
  }

  return fileObject;
}