"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fileList = fileList;
exports.grabDataFrom = grabDataFrom;
exports.grabDataFromSource = grabDataFromSource;
exports.removeDataList = removeDataList;
exports.removeNonEachDataList = removeNonEachDataList;
exports.removeNonEditableDataList = removeNonEditableDataList;
exports.requestList = requestList;
exports.updateJsonWithExternalData = updateJsonWithExternalData;
exports.urlList = urlList;
exports.valueList = valueList;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _bluebird = require("bluebird");

var _http = _interopRequireDefault(require("http"));

var _https = _interopRequireDefault(require("https"));

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _lodash = _interopRequireDefault(require("lodash"));

var _2 = require("../../");

function requestList(obj, match, jsonPage) {
  var p = new _bluebird.Promise(function (resolve, reject) {
    _2.cmsData.sql.executeQuery(match, jsonPage).then(function (data) {
      jsonPage["abe_source"] = jsonPage["abe_source"] || {};
      jsonPage["abe_source"][obj.key] = data; // I update the jsonPage[obj.key] when the tag is not editable

      if (!obj.editable) {
        if (obj["max-length"]) {
          jsonPage[obj.key] = data.slice(0, obj["max-length"]);
        } else {
          jsonPage[obj.key] = data;
        } // I update the jsonPage[obj.key] only if empty or obj.prefill

      } else if (jsonPage[obj.key] == null && obj.prefill) {
        if (obj["prefill-quantity"] && obj["max-length"]) {
          jsonPage[obj.key] = data.slice(0, obj["prefill-quantity"] > obj["max-length"] ? obj["max-length"] : obj["prefill-quantity"]);
        } else if (obj["prefill-quantity"]) {
          jsonPage[obj.key] = data.slice(0, obj["prefill-quantity"]);
        } else if (obj["max-length"]) {
          jsonPage[obj.key] = data.slice(0, obj["max-length"]);
        } else {
          jsonPage[obj.key] = data;
        }
      }

      resolve(jsonPage);
    })["catch"](function (e) {
      console.log('e', e);
      reject(e);
    });
  });
  return p;
}

function valueList(obj, match, jsonPage) {
  var p = new _bluebird.Promise(function (resolve) {
    var value = _2.cmsData.sql.getDataSource(match);

    if (value.indexOf("{") > -1 || value.indexOf("[") > -1) {
      try {
        value = JSON.parse(value);
        jsonPage["abe_source"][obj.key] = value;
      } catch (e) {
        jsonPage["abe_source"][obj.key] = null;
        console.log("Error ".concat(value, "/is not a valid JSON"), "\n".concat(e));
      }
    }

    resolve();
  });
  return p;
}
/**
 * Sets in jsonPage["abe_source"][obj.key] the data grabbed from a HTTP call
 * or just the URL if attribute autocomplete == true
 *
 * @param {*} obj
 * @param {*} match
 * @param {*} jsonPage
 * @returns
 */


function urlList(abeTag, match, jsonPage) {
  var p = new _bluebird.Promise(function (resolve) {
    if (abeTag.autocomplete !== true && abeTag.autocomplete !== "true") {
      var host = abeTag.sourceString;
      host = host.split("/");
      var httpUse = _http["default"];
      var defaultPort = 80;

      if (host[0] === "https:") {
        httpUse = _https["default"];
        defaultPort = 443;
      }

      host = host[2].split(":");
      var pathSource = abeTag.sourceString.split("//");

      if (pathSource[1] != null) {
        pathSource = pathSource[1].split("/");
        pathSource.shift();
        pathSource = "/" + pathSource.join("/");
      } else {
        pathSource = "/";
      }

      var options = {
        hostname: host[0],
        port: host[1] != null ? host[1] : defaultPort,
        path: pathSource,
        method: "GET",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "Content-Length": 0
        }
      };
      var body = "";
      var localReq = httpUse.request(options, function (localRes) {
        localRes.setEncoding("utf8");
        localRes.on("data", function (chunk) {
          body += chunk;
        });
        localRes.on("end", function () {
          try {
            if (typeof body === "string") {
              var parsedBody = JSON.parse(body);

              if ((0, _typeof2["default"])(parsedBody) === "object" && Object.prototype.toString.call(parsedBody) === "[object Array]") {
                jsonPage["abe_source"][abeTag.key] = parsedBody;
              } else if ((0, _typeof2["default"])(parsedBody) === "object" && Object.prototype.toString.call(parsedBody) === "[object Object]") {
                jsonPage["abe_source"][abeTag.key] = [parsedBody];
              }
            } else if ((0, _typeof2["default"])(body) === "object" && Object.prototype.toString.call(body) === "[object Array]") {
              jsonPage["abe_source"][abeTag.key] = body;
            } else if ((0, _typeof2["default"])(body) === "object" && Object.prototype.toString.call(body) === "[object Object]") {
              jsonPage["abe_source"][abeTag.key] = body;
            }
          } catch (e) {
            console.log("Error ".concat(abeTag.sourceString, " is not a valid JSON"), "\n".concat(e));
          }

          resolve();
        });
      });
      localReq.on("error", function (e) {
        console.log(e);
      }); // write data to request body

      localReq.write("");
      localReq.end();
    } else {
      jsonPage["abe_source"][abeTag.key] = abeTag.sourceString;
      resolve();
    }
  });
  return p;
}
/**
 * Sets in jsonPage["abe_source"][abeTag.key] the content
 * from a local file
 *
 * @param Object abeTag
 * @param String jsonPage
 * @returns
 */


function fileList(abeTag, jsonPage) {
  var p = new _bluebird.Promise(function (resolve) {
    var filePath = abeTag.sourceString;

    if (filePath.charAt(0) == "/") {
      filePath = _path["default"].join(_2.config.root, filePath);
    } else {
      filePath = _path["default"].join(_2.config.root, _2.config.reference.url, filePath);
    }

    jsonPage["abe_source"][abeTag.key] = _2.coreUtils.file.getJson(filePath);
    resolve();
  });
  return p;
}

function grabDataFromSource(_x, _x2) {
  return _grabDataFromSource.apply(this, arguments);
}
/**
 * Parses an abe type 'data' and returns the related data
 * in the jsonPage's 'abe_source' attribute
 * depending on the nature of the source.
 * @param String jsonPage
 * @param String tagStr represents the abe tag
 * @returns
 */


function _grabDataFromSource() {
  _grabDataFromSource = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(jsonPage, match) {
    var obj, sourceType, filePath, newJson, _newJson, _newJson2;

    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            obj = _2.cmsData.attributes.getAll(match, jsonPage);
            sourceType = _2.cmsData.sql.getSourceType(obj.sourceString);

            if (!(sourceType === "file" && obj.type !== "data" && obj.type !== "import")) {
              _context.next = 31;
              break;
            }

            obj = _2.cmsData.attributes.sanitizeSourceAttribute(obj, jsonPage);
            filePath = obj.sourceString;

            if (filePath.charAt(0) == "/") {
              filePath = _path["default"].join(_2.config.root, filePath);
            } else {
              filePath = _path["default"].join(_2.config.root, _2.config.reference.url, filePath);
            } // TODO: What is this following case for ?
            // Why a key in the original json copied to another one
            // then swiped from the original json ?
            // Do I have to save data


            if (!(_lodash["default"].get(jsonPage, obj.key) != null)) {
              _context.next = 26;
              break;
            }

            newJson = {};

            if (!_2.coreUtils.file.exist(filePath)) {
              _context.next = 18;
              break;
            }

            _context.next = 11;
            return _2.cmsData.file.get(filePath);

          case 11:
            _newJson = _context.sent;

            if (!(_lodash["default"].get(_newJson, obj.key) !== _lodash["default"].get(jsonPage, obj.key))) {
              _context.next = 16;
              break;
            }

            _lodash["default"].set(_newJson, obj.key, _lodash["default"].get(jsonPage, obj.key));

            _context.next = 16;
            return _fsExtra["default"].writeJson(filePath, _newJson, {
              space: 2,
              encoding: "utf-8"
            });

          case 16:
            _context.next = 23;
            break;

          case 18:
            _lodash["default"].set(newJson, obj.key, _lodash["default"].get(jsonPage, obj.key));

            _context.next = 21;
            return _fsExtra["default"].mkdir(filePath.split("/").slice(0, -1).join("/"));

          case 21:
            _context.next = 23;
            return _fsExtra["default"].writeJson(filePath, newJson, {
              space: 2,
              encoding: "utf-8"
            });

          case 23:
            _lodash["default"].unset(jsonPage, obj.key); // Or do I have to only read data


            _context.next = 31;
            break;

          case 26:
            if (!_2.coreUtils.file.exist(filePath)) {
              _context.next = 31;
              break;
            }

            _context.next = 29;
            return _2.cmsData.file.get(filePath);

          case 29:
            _newJson2 = _context.sent;

            if (_lodash["default"].get(_newJson2, obj.key) != null) {
              _lodash["default"].set(jsonPage, obj.key, _lodash["default"].get(_newJson2, obj.key));
            }

          case 31:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _grabDataFromSource.apply(this, arguments);
}

function grabDataFrom(jsonPage, tagStr) {
  var p = new _bluebird.Promise(function (resolve) {
    jsonPage["abe_source"] = jsonPage["abe_source"] || {};

    var obj = _2.cmsData.attributes.getAll(tagStr, jsonPage);

    var type = _2.cmsData.sql.getSourceType(obj.sourceString);

    switch (type) {
      case "request":
        obj = _2.cmsData.attributes.sanitizeSourceAttribute(obj, jsonPage);
        requestList(obj, tagStr, jsonPage).then(function (jsonPage) {
          resolve(jsonPage);
        })["catch"](function (e) {
          console.log("[ERROR] source.js requestList", e);
        });
        break;

      case "value":
        obj = _2.cmsData.attributes.sanitizeSourceAttribute(obj, jsonPage);
        valueList(obj, tagStr, jsonPage).then(function () {
          resolve();
        })["catch"](function (e) {
          console.log("[ERROR] source.js valueList", e);
        });
        break;

      case "url":
        urlList(obj, tagStr, jsonPage).then(function () {
          resolve();
        })["catch"](function (e) {
          console.log("[ERROR] source.js urlList", e);
        });
        break;

      case "file":
        obj = _2.cmsData.attributes.sanitizeSourceAttribute(obj, jsonPage);
        fileList(obj, jsonPage).then(function () {
          resolve();
        })["catch"](function (e) {
          console.log("[ERROR] source.js fileList", e);
        });
        break;

      default:
        resolve();
        break;
    }
  });
  return p;
}
/**
 * Parse the Abe tags with type 'data' and fill the
 * JsonPage['abe_source'] with grabbed data
 * then updates the JsonPage attributes with data
 * from external sources (not in abe type 'import' nor 'data')
 *
 * @param String text
 * @param String jsonPage
 * @returns
 */


function updateJsonWithExternalData(_x3, _x4) {
  return _updateJsonWithExternalData.apply(this, arguments);
}

function _updateJsonWithExternalData() {
  _updateJsonWithExternalData = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(text, jsonPage) {
    var dataTypes, externalSources;
    return _regenerator["default"].wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            dataTypes = _2.cmsData.regex.getAbeTypeDataList(text);
            _context4.next = 3;
            return _bluebird.Promise.all(dataTypes.map( /*#__PURE__*/function () {
              var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(tagStr) {
                return _regenerator["default"].wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        _context2.next = 2;
                        return grabDataFrom(jsonPage, tagStr);

                      case 2:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, _callee2);
              }));

              return function (_x5) {
                return _ref.apply(this, arguments);
              };
            }()));

          case 3:
            externalSources = _2.cmsData.regex.getTagAbeWithSource(text);
            _context4.next = 6;
            return _bluebird.Promise.all(externalSources.map( /*#__PURE__*/function () {
              var _ref2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(tagStr) {
                return _regenerator["default"].wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        _context3.next = 2;
                        return grabDataFromSource(jsonPage, tagStr);

                      case 2:
                      case "end":
                        return _context3.stop();
                    }
                  }
                }, _callee3);
              }));

              return function (_x6) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 6:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return _updateJsonWithExternalData.apply(this, arguments);
}

function removeDataList(text) {
  var newText = text.replace(_2.cmsData.regex.dataTypeRegWithComment, "");
  newText = newText.replace(_2.cmsData.regex.dataTypeReg, "");
  return newText;
}

function removeNonEditableDataList(text) {
  var newText = text.replace(_2.cmsData.regex.nonEditableDataReg, "");
  return newText;
}
/**
 * Remove all type='data' outside {{#each}} statements
 * @param  {[type]} text [description]
 * @return {[type]}      [description]
 */


function removeNonEachDataList(text) {
  // removing each blocks potentially containing abe data type
  var pattEach = /(\{\{#each (\r|\t|\n|.)*?\/each\}\})/g;
  var textWithNoEach = text.replace(pattEach, "");
  var match;

  while (match = _2.cmsData.regex.dataTypeReg.exec(textWithNoEach)) {
    text = text.replace(match[0], "");
  }

  return text;
}