"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.escapeAbeValuesFromStringRequest = escapeAbeValuesFromStringRequest;
exports.execQuery = execQuery;
exports.executeFromClause = executeFromClause;
exports.executeLimitClause = executeLimitClause;
exports.executeOrderByClause = executeOrderByClause;
exports.executeQuery = executeQuery;
exports.executeQuerySync = executeQuerySync;
exports.executeWhereClause = executeWhereClause;
exports.getDataSource = getDataSource;
exports.getFromDirectory = getFromDirectory;
exports.getSourceType = getSourceType;
exports.getWhereValuesToCompare = getWhereValuesToCompare;
exports.handleSqlRequest = handleSqlRequest;
exports.isInStatementCorrect = isInStatementCorrect;
exports.keepOnlyPublishedPost = keepOnlyPublishedPost;
exports.recurseWhere = recurseWhere;
exports.sanitizeFromStatement = sanitizeFromStatement;

var _nodeSqlparser = require("node-sqlparser");

var _bluebird = require("bluebird");

var _path = _interopRequireDefault(require("path"));

var _lodash = _interopRequireDefault(require("lodash"));

var _2 = require("../../");

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * take a string and json to escape sql character and convert to sql like syntax
 *
 * Example: escapeAbeValuesFromStringRequest('select title from ./ where `abe_meta.template`=`{{article}}`', {article: "test"})
 *
 * Return string: select title from ___abe_dot______abe_dot______abe___ where  `abe_meta.template`=`test`
 *
 *
 * @param  {String} str      raw abe request sql string
 * @param  {Object} jsonPage json object of post
 * @return {String}          escaped string
 */
function escapeAbeValuesFromStringRequest(str, jsonPage) {
  var matchFrom = /from .(.*?) /;
  var matchVariable = /{{(.*?)}}/;
  var matchFromExec = matchFrom.exec(str);

  if (matchFromExec != null && matchFromExec[1] != null) {
    var fromMatch;
    var toReplace = matchFromExec[1];

    while (fromMatch = matchVariable.exec(toReplace)) {
      try {
        var arFrom = fromMatch[1].split('.');

        var value = _lodash["default"].get(jsonPage, arFrom);

        if (value != null) {
          toReplace = toReplace.replace('{{' + fromMatch[1] + '}}', value);
        } else {
          toReplace = toReplace.replace('{{' + fromMatch[1] + '}}', '');
        }
      } catch (e) {}
    }

    str = str.replace(matchFromExec[1], toReplace);
  }

  var from = /from ([\S\s]+)/.exec(str);
  var matches = from;

  if (matches[1]) {
    var res = matches[1];
    var splitAttr = [' where ', ' order by ', ' limit ', ' WHERE ', ' ORDER BY ', ' LIMIT '];

    for (var i = 0; i < splitAttr.length; i++) {
      if (res.indexOf(splitAttr[i]) > -1) {
        res = res.substring(0, res.indexOf(splitAttr[i]));
      }
    }

    var escapedFrom = res.replace(/\//g, '___abe___');
    escapedFrom = escapedFrom.replace(/\./g, '___abe_dot___');
    escapedFrom = escapedFrom.replace(/-/g, '___abe_dash___');
    str = str.replace(res, escapedFrom);
  }

  str = str.replace(/``/g, "''");
  return str;
}
/**
 * analyse and create an object from request sql string
 *
 * Example: handleSqlRequest('select title from ./ where `abe_meta.template`=`{{article}}`', {article: "test"})
 *
 * @param  {String} str      Sql string request
 * @param  {Object} jsonPage json of post
 * @return {Object}          {type, columns, from, where, string, limit, orderby}
 */


function handleSqlRequest(str, jsonPage) {
  var req = escapeAbeValuesFromStringRequest(str, jsonPage);
  var request = (0, _nodeSqlparser.parse)(req);
  var reconstructSql = ''; // SQL TYPE

  var type = '';

  if (request.type != null) {
    type = request.type;
  }

  reconstructSql += "".concat(type, " "); // SQL COLUMNS

  var columns = [];

  if (request.columns != null) {
    if (request.columns === '*') {
      columns.push('*');
    } else {
      Array.prototype.forEach.call(request.columns, function (item) {
        if (_lodash["default"].get(item, 'expr.type') === 'binary_expr') {
          columns.push("".concat(item.expr.left.column).concat(item.expr.operator).concat(item.expr.right.column));
        } else {
          columns.push(item.expr.column);
        }
      });
    }
  }

  reconstructSql += "".concat(JSON.stringify(columns), " "); // SQL FROM

  var from = [];

  if (request.from != null) {
    Array.prototype.forEach.call(request.from, function (item) {
      from.push(item.table);
    });
  } else {
    from.push('*');
  }

  reconstructSql += "from ".concat(JSON.stringify(from), " ");
  var where = null;

  if (request.where != null) {
    where = request.where;
  }

  var limit = -1;

  if (request.limit != null) {
    limit = request.limit[request.limit.length - 1].value;
  }

  var orderby = null;

  if (request.orderby != null && request.orderby.length > 0) {
    orderby = {
      column: request.orderby[0].expr.column,
      type: request.orderby[0].type
    };
    reconstructSql += "ORDER BY ".concat(orderby.column, " ").concat(orderby.type, " ");
  }

  return {
    type: type,
    columns: columns,
    from: from,
    where: where,
    string: reconstructSql,
    limit: limit,
    orderby: orderby
  };
}
/**
 * get JSON from abe tag attribute source
 *
 * {{abe type='data' key='titles' desc='select titles' source='[{"title": "rouge", "id": 1},{"title": "vert", "id": 2},{"title": "blue", "id": 3}]' display="{{title}}"}}
 *
 * return
 * [{"title": "rouge", "id": 1},{"title": "vert", "id": 2},{"title": "blue", "id": 3}]
 *
 * @param  {String} str abe tag
 * @return {String}     json string
 */


function getDataSource(str) {
  var reg = /source=(['|"])(.*?)\1[ |\}]/g;
  var match = reg.exec(str);

  if (match != null) {
    return match[2];
  }

  return '';
}
/**
 * replaces escaped characters with the right ones
 * @param  {String} statement the from clause
 * @return {String}           the from sanitized
 */


function sanitizeFromStatement(statement) {
  var from = '';

  if (statement != null) {
    from = statement[0].replace(/___abe_dot___/g, '.');
    from = from.replace(/___abe___/g, '/');
    from = from.replace(/___abe_dash___/g, '-');
  }

  return from;
}
/**
 * calculate the directory to analyze from the from clause
 * @param  {String} statement the from clause
 * @param  {String} tplPath   the path from the template originator
 * @return {string}           the directory to analyze
 */


function getFromDirectory(statement) {
  var tplPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '/';
  var pathFromDir = '';

  if (statement === '' || statement === '*' || statement === '/') {
    pathFromDir = _2.Manager.instance.pathData;
  } else if (statement === './') {
    pathFromDir = _path["default"].join(_2.Manager.instance.pathData, tplPath);
  } else if (statement.indexOf('/') === 0) {
    pathFromDir = _path["default"].join(_2.Manager.instance.pathData, statement);
  } else if (statement.indexOf('/') !== 0) {
    pathFromDir = _path["default"].join(_2.Manager.instance.pathData, tplPath, statement);
  }

  return pathFromDir;
}
/**
 * sort array of files from where clause
 *
 * @param  {Array} files
 * @param  {Object} orderby {orderby: {column: 'date'}} | {orderby: {column: 'random', type: 'ASC'}}
 * @return {Array}         sorted array
 */


function executeOrderByClause(files, orderby) {
  if (orderby != null) {
    if (orderby.column.toLowerCase() === 'random') {
      files = _2.coreUtils.sort.shuffle(files);
    } else if (orderby.column.toLowerCase() === 'date') {
      if (orderby.type.toLowerCase() === 'asc') {
        files.sort(_2.cmsData.sort.byDateAsc);
      } else if (orderby.type.toLowerCase() === 'desc') {
        files.sort(_2.cmsData.sort.byDateDesc);
      }
    } else {
      files.sort(_2.coreUtils.sort.predicatBy(orderby.column.toLowerCase(), orderby.type.toLowerCase() === 'desc' ? -1 : 1));
    }
  }

  return files;
}
/**
 * Keep only published post
 *
 * keepOnlyPublishedPost([files])
 *
 * @param  {Array} files      paths
 * @return {Array}                files
 */


function keepOnlyPublishedPost(files) {
  var publishedValue = [];

  if (files && files.length > 0) {
    Array.prototype.forEach.call(files, function (file) {
      if (file.publish != null) {
        publishedValue.push(file.publish);
      }
    });
  }

  return publishedValue;
}
/**
 * Check array of files have path that match path statement
 *
 * executeFromClause([array], ['/'], ['/'])
 *
 * @param  {Array} statement      paths
 * @param  {Array} pathFromClause paths
 * @return {Array}                files
 */


function executeFromClause(files, statement, pathFromClause) {
  var from = sanitizeFromStatement(statement); // if the from clause ends with a dot, we won't recurse the directory analyze

  if (from.slice(-1) === '.') {
    from = from.slice(0, -1);
  }

  var fromDirectory = getFromDirectory(from, pathFromClause);
  var files_array = files.filter(function (element) {
    if (element.path.indexOf(fromDirectory) > -1) {
      return true;
    }

    return false;
  });
  return files_array;
}
/**
 * Execute sql query like to find abe json post that match the query
 *
 * @param  {Array} pathQuery of paths
 * @param  {String} match     request sql
 * @param  {Object} jsonPage  json of post
 * @return {Array}           found object that match
 */


function execQuery(match, jsonPage) {
  var files = keepOnlyPublishedPost(_2.Manager.instance.getList());
  var request = handleSqlRequest(_2.cmsData.regex.getAttr(match, 'source'), jsonPage);
  var pathQuery = jsonPage && jsonPage.abe_meta != null && jsonPage.abe_meta.link != null ? _path["default"].dirname(jsonPage.abe_meta.link) : '/';
  files = executeFromClause(files, request.from, pathQuery);
  files = executeWhereClause(files, request.where, request.columns, jsonPage);
  files = executeOrderByClause(files, request.orderby);
  files = executeLimitClause(files, request.limit);
  return files;
}

function executeQuerySync(match, jsonPage) {
  return execQuery(match, jsonPage);
}

function executeQuery(match, jsonPage) {
  var p = new _bluebird.Promise(function (resolve) {
    var res = execQuery(match, jsonPage);
    resolve(res);
  })["catch"](function (e) {
    console.error(e);
  });
  return p;
}
/**
 * check if a given string an url, string json, file url, abe sql request
 *
 * get('http://google.com')
 * get('{"test":"test"}')
 * get('select * from ../')
 * get('test')
 *
 * @param  {String} str
 * @return {String} url | request | value | file | other
 */


function getSourceType(str) {
  if (/^(http:\/\/|https:\/\/|:\/\/|\/(.*)\?(.*)[^ ]=)/.test(str)) {
    return 'url';
  }

  if (/select[\S\s]*?from/.test(str)) {
    return 'request';
  }

  try {
    JSON.parse(str);
    return 'value';
  } catch (e) {}

  if (/\.json/.test(str)) {
    return 'file';
  }

  return 'other';
}
/**
 * return array of post that match sql where statement
 *
 * Example: handleSqlRequest('select title from ./ where `abe_meta.template`=`article`', {})
 *
 * @param  {Array} files
 * @param  {Object} wheres   clause
 * @param  {Int} maxLimit
 * @param  {Array} columns  sql
 * @param  {Object} jsonPage json post
 * @return {Array}          of files
 */


function executeWhereClause(files, wheres, columns, jsonPage) {
  if (typeof wheres === 'undefined' || wheres === null) return files;
  var res = [];
  var json = {};
  var jsonValues = {};

  var _iterator = _createForOfIteratorHelper(files),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var file = _step.value;

      if (wheres != null) {
        if (!recurseWhere(wheres, file, jsonPage)) {
          json = JSON.parse(JSON.stringify(file));
          jsonValues = {};

          if (columns != null && columns.length > 0 && columns[0] !== '*') {
            Array.prototype.forEach.call(columns, function (column) {
              if (json[column] != null) {
                jsonValues[column] = json[column];
              }
            });
            jsonValues['abe_meta'] = json['abe_meta'];
          } else {
            jsonValues = json;
          }

          res.push(jsonValues);
        }
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return res;
}
/**
 * return array of post in the limit
 *
 * Example: executeLimitClause({}, 2)
 *
 * @param  {Array} files
 * @param  {Int} maxLimit
 * @return {Array} of files
 */


function executeLimitClause(files, maxLimit) {
  if (files.length > maxLimit && maxLimit > 0) {
    files.splice(maxLimit, files.length - maxLimit);
  }

  return files;
}
/**
 * Compare where left and where right clause
 *
 * @param  {Object} where           clause
 * @param  {Object} jsonDoc         json of current post
 * @param  {Object} jsonOriginalDoc json of post to compare
 * @return {Object}                 {left: value, right: value}
 */


function getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc) {
  var regexIsVariable = /^{{(.*)}}$/;
  var value = null;
  var compare = null;

  try {
    var variableLeft = where.left.column;
    var whereLeftColumn = where.left.column;
    var checkIfLeftIsAVariable = regexIsVariable.exec(variableLeft);

    if (checkIfLeftIsAVariable != null && checkIfLeftIsAVariable.length > 0) {
      variableLeft = checkIfLeftIsAVariable[1];
    }

    var arVariableLeft = variableLeft.split('.');
    value = _lodash["default"].get(jsonDoc, arVariableLeft);
  } catch (e) {// console.log('e', e)
  }

  if (where.operator === 'IN' || where.operator === 'NOT IN') {
    if (value == null) {
      // is the form of variableLeft something like elt.attribute or elt[].attribute?
      // so maybe is elt an array ?
      var arValues = variableLeft.split('.');
      var key = arValues[0];

      if (key.slice(-2) == '[]') {
        key = key.slice(0, -2);
      }

      var records = _lodash["default"].get(jsonDoc, key);

      whereLeftColumn = arValues[1];
      value = []; // if yes, value is then an array of values

      if (records != null) {
        Array.prototype.forEach.call(records, function (record) {
          try {
            var val = record[arValues[1]];
            value.push(val);
          } catch (e) {
            console.log(e.stack);
          }
        });
      }
    }

    compare = [];
    Array.prototype.forEach.call(where.right.value, function (right) {
      var matchRightVariable = regexIsVariable.exec(right.column);

      if (matchRightVariable != null && matchRightVariable.length > 0) {
        try {
          var jsonOriginalValues = _lodash["default"].get(jsonOriginalDoc, matchRightVariable[1]);

          if (Object.prototype.toString.call(jsonOriginalValues) === '[object Array]') {
            Array.prototype.forEach.call(jsonOriginalValues, function (jsonOriginalValue) {
              compare.push(_lodash["default"].get(jsonOriginalValue, whereLeftColumn));
            });
          } else {
            compare.push(jsonOriginalValues);
          }
        } catch (e) {}
      } else {
        compare.push(right.column);
      }
    });
  } else {
    if (where.right.column != null) {
      compare = where.right.column;
    } else if (where.right.value != null) {
      compare = where.right.value;
    }

    var matchRightVariable = regexIsVariable.exec(compare);

    if (matchRightVariable != null && matchRightVariable.length > 0) {
      try {
        var arVariableRight = matchRightVariable[1].split('.');

        var shouldCompare = _lodash["default"].get(jsonOriginalDoc, arVariableRight);

        if (shouldCompare != null) {
          compare = shouldCompare;
        } else {
          compare = null;
        }
      } catch (e) {
        compare = null;
      }
    }
  }

  return {
    left: value,
    right: compare
  };
}

function isInStatementCorrect(values, isCorrect) {
  var hasPassedTest = false;

  if (Object.prototype.toString.call(values.left) === '[object Array]') {
    Array.prototype.forEach.call(values.left, function (left) {
      Array.prototype.forEach.call(values.right, function (right) {
        if (left != null && left === right) {
          hasPassedTest = true;
        }
      });
    });
  } else {
    Array.prototype.forEach.call(values.right, function (right) {
      if (values.left === right) {
        hasPassedTest = true;
      }
    });
  }

  if (hasPassedTest) isCorrect = !isCorrect;
  return isCorrect;
}
/**
 * Check where.left value that match where operator (=, !=, >, >=, <, <=, LIKE, NOT LIKE, AND, OR, IN, NOT IN)
 * if operator AND or OR
 * Recurse on where.left and where.right sql clause
 *
 *
 * @param  {Object} where           clause
 * @param  {Object} jsonDoc         json of current post
 * @param  {Object} jsonOriginalDoc json of post to compare
 * @return {Boolean}                 true if not matching | false if matching
 */


function recurseWhere(where, jsonDoc, jsonOriginalDoc) {
  var isNotLeftCorrect = false;
  var isNotRightCorrect = false;
  var isNotCorrect = false;
  var values;

  switch (where.operator) {
    case '=':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left === values.right);
      break;

    case '!=':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left !== values.right);
      break;

    case '>':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left > values.right);
      break;

    case '>=':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left >= values.right);
      break;

    case '<':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left < values.right);
      break;

    case '<=':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left <= values.right);
      break;

    case 'LIKE':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left && values.left.indexOf(values.right) > -1);
      break;

    case 'NOT LIKE':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = !(values.left && values.left.indexOf(values.right) === -1);
      break;

    case 'AND':
      isNotLeftCorrect = recurseWhere(where.left, jsonDoc, jsonOriginalDoc);
      isNotRightCorrect = recurseWhere(where.right, jsonDoc, jsonOriginalDoc);
      isNotCorrect = isNotLeftCorrect || isNotRightCorrect ? true : false;
      break;

    case 'OR':
      isNotLeftCorrect = recurseWhere(where.left, jsonDoc, jsonOriginalDoc);
      isNotRightCorrect = recurseWhere(where.right, jsonDoc, jsonOriginalDoc);
      isNotCorrect = isNotLeftCorrect && isNotRightCorrect ? true : false;
      break;

    case 'IN':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = isInStatementCorrect(values, true);
      break;

    case 'NOT IN':
      values = getWhereValuesToCompare(where, jsonDoc, jsonOriginalDoc);
      isNotCorrect = isInStatementCorrect(values, false);
      break;
  }

  return isNotCorrect;
}