"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDocPath = getDocPath;
exports.getDocPathFromLinkStatusDate = getDocPathFromLinkStatusDate;
exports.getDocPathFromPostUrl = getDocPathFromPostUrl;
exports.getDocRelativePath = getDocRelativePath;
exports.getDocRelativePathFromPostUrl = getDocRelativePathFromPostUrl;
exports.getPercentOfRequiredTagsFilled = getPercentOfRequiredTagsFilled;
exports.getPostPath = getPostPath;
exports.getPostRelativePath = getPostRelativePath;
exports.getPostUrl = getPostUrl;
exports.getRevisionPath = getRevisionPath;
exports.getRevisionRelativePath = getRevisionRelativePath;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _path = _interopRequireDefault(require("path"));

var _ = require("../../");

function getPercentOfRequiredTagsFilled(text, json) {
  var regAbe = /{{abe[\S\s].*?key=['|"]([\S\s].*?['|"| ]}})/g;
  var matches = text.match(regAbe);
  var requiredValue = 0;
  var complete = 0;

  if (typeof matches !== 'undefined' && matches !== null) {
    Array.prototype.forEach.call(matches, function (match) {
      if (typeof match !== 'undefined' && match !== null) {
        var keyAttr = _.cmsData.regex.getAttr(match, 'key');

        var requiredAttr = _.cmsData.regex.getAttr(match, 'required');

        if (requiredAttr === 'true') {
          requiredValue++;

          var minAttr = _.cmsData.regex.getAttr(match, 'min-length');

          minAttr = minAttr !== '' ? minAttr : 0;

          if (typeof json[keyAttr] !== 'undefined' && json[keyAttr] !== null && json[keyAttr] !== '') {
            if (minAttr > 0) {
              if (json[keyAttr].length >= minAttr) {
                complete++;
              }
            } else {
              complete++;
            }
          }
        }
      }
    });
  }

  return Math.round(requiredValue > 0 ? complete * 100 / requiredValue : 100);
}
/**
 * Adds the data absolute path to the jsonPath
 * ex. jsonPath = fr/test-abe-d20170418T130602280Z.json
 * return /Users/grg/programmation/git/abetesttheme/data/fr/test-abe-d20170418T130602280Z.json
 * and returns this relative path to the json revision
 * @param  {[type]} jsonPath [description]
 * @return {[type]}          [description]
 */


function getRevisionPath(jsonPath) {
  return _path["default"].join(_.Manager.instance.pathData, getRevisionRelativePath(jsonPath));
}
/**
 * Remove the data absolute path from the jsonPath
 * ex. jsonPath = /Users/grg/programmation/git/abetesttheme/data/fr/test-abe-d20170418T130602280Z.json
 * return fr/test-abe-d20170418T130602280Z.json
 * and returns this relative path to the json revision
 * @param  {[type]} jsonPath [description]
 * @return {[type]}          [description]
 */


function getRevisionRelativePath(jsonPath) {
  return jsonPath.replace(_.Manager.instance.pathData + _path["default"].sep, '');
}
/**
 * Return the absolute path of the doc from the post URL
 * @param  {[type]} postUrl [description]
 * @return {[type]}         [description]
 */


function getDocPathFromPostUrl(postUrl) {
  var extension = '.json';
  var templateExtension = '.' + _.config.files.templates.extension;
  var osJsonPath = postUrl.replace(templateExtension, extension).replace('/', _path["default"].sep);
  return _path["default"].join(_.Manager.instance.pathData, osJsonPath);
}
/**
 * Return the relative path of the doc from the post URL
 * @param  {[type]} postUrl [description]
 * @return {[type]}         [description]
 */


function getDocRelativePathFromPostUrl(postUrl) {
  var docPath = getDocPathFromPostUrl(postUrl);
  return getDocRelativePath(docPath);
}
/**
 * Return the absolute path of the doc from a post URL, a status and a date
 * @param  {[type]} postUrl [description]
 * @return {[type]}         [description]
 */


function getDocPathFromLinkStatusDate(jsonData) {
  var jsonObject = _.cmsData.file.getAbeMeta({}, jsonData);

  var dateIso = jsonObject.abe_meta.date;

  if ((0, _typeof2["default"])(dateIso) === 'object') {
    dateIso = dateIso.toISOString();
  }

  var statusDate = jsonObject.abe_meta.status[0] + _.cmsData.revision.removeStatusAndDateFromFileName(dateIso);

  return getDocPathFromPostUrl(_.cmsData.fileAttr.add(jsonObject.abe_meta.link, statusDate));
}
/**
 * Remove the date revision from the jsonPath and returns the doc full path (which is the published json)
 * ex. jsonPath = /Users/grg/programmation/git/abetesttheme/data/fr/test-abe-d20170418T130602280Z.json
 * return /Users/grg/programmation/git/abetesttheme/data/fr/test.json
 * @param  {[type]} jsonPath [description]
 * @return {[type]}          [description]
 */


function getDocPath(jsonPath) {
  return _.cmsData.fileAttr["delete"](jsonPath);
}
/**
 * Remove the date revision from the jsonPath and returns the doc relative path (which is the published json)
 * ex. jsonPath = /Users/grg/programmation/git/abetesttheme/data/fr/test-abe-d20170418T130602280Z.json
 * return fr/test.json
 * @param  {[type]} jsonPath [description]
 * @return {[type]}          [description]
 */


function getDocRelativePath(jsonPath) {
  return _.cmsData.fileAttr["delete"](getRevisionRelativePath(jsonPath));
}
/**
 * Remove the date revision from the jsonPath and returns the html full path (which is the published post)
 * ex. jsonPath = /Users/grg/programmation/git/abetesttheme/data/fr/test-abe-d20170418T130602280Z.json
 * return /Users/grg/programmation/git/abetesttheme/site/fr/test.html
 * can be:
 * - the full path to the revision : /Users/myname/myAbeBlog/data/fr/abe/is/awesome-d-20170808020202TZ.json
 * - the relative path to the revision : fr/abe/is/awesome-d-20170808020202TZ.json
 * - the full path to the doc : /Users/myname/myAbeBlog/data/fr/abe/is/awesome.json
 * - the relative path to the doc : fr/abe/is/awesome.json
 * - the relative path to the post : fr/abe/is/awesome.html
 *
 * will return : /Users/myname/myAbeBlog/site/fr/abe/is/awesome.html
 * 
 * @param  {[type]} jsonPath [description]
 * @return {[type]}          [description]
 */


function getPostPath(jsonPath) {
  return _path["default"].join(_.Manager.instance.pathPublish, getPostRelativePath(jsonPath));
}
/**
 * Remove the date revision from the jsonPath and returns the html relative path (which is the published post)
 * ex. jsonPath = /Users/grg/programmation/git/abetesttheme/data/fr/test-abe-d20170418T130602280Z.json
 * return fr/test.html (under window it could be fr\test.html)
 * @param  {[type]} jsonPath [description]
 * @return {[type]}          [description]
 */


function getPostRelativePath(jsonPath) {
  var extension = '.json';
  var templateExtension = '.' + _.config.files.templates.extension;
  return getDocRelativePath(jsonPath).replace(extension, templateExtension);
}
/**
 * Remove the date revision from the jsonPath and returns the URL absolute path of the published post)
 * ex. jsonPath = /Users/grg/programmation/git/abetesttheme/data/fr/test-abe-d20170418T130602280Z.json
 * return fr/test.html
 * @param  {[type]} jsonPath [description]
 * @return {[type]}          [description]
 */


function getPostUrl(jsonPath) {
  var extension = '.json';
  var templateExtension = '.' + _.config.files.templates.extension;
  return _path["default"].join('/', getPostRelativePath(jsonPath).replace(extension, templateExtension));
}