"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMediaFolder = createMediaFolder;
exports.createMediaSlug = createMediaSlug;
exports.cropAndSaveFile = cropAndSaveFile;
exports.cropAndSaveFiles = cropAndSaveFiles;
exports.generateThumbnail = void 0;
exports.getAssociatedImageFileFromThumb = getAssociatedImageFileFromThumb;
exports.getMediaType = getMediaType;
exports.getThumbsList = getThumbsList;
exports.isValidMedia = isValidMedia;
exports.saveFile = saveFile;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _childProcessPromise = _interopRequireDefault(require("child-process-promise"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _fs = _interopRequireDefault(require("fs"));

var _slugify = _interopRequireDefault(require("slugify"));

var _jimp = _interopRequireDefault(require("jimp"));

var _path = _interopRequireDefault(require("path"));

var _bluebird = require("bluebird");

var _smartcropJimp = _interopRequireDefault(require("smartcrop-jimp"));

var _ = require("../../");

function cropAndSaveFile(imageSize, file, newFile) {
  var p = new _bluebird.Promise(function (resolve) {
    _smartcropJimp["default"].crop(file, {
      width: parseInt(imageSize[0]),
      height: parseInt(imageSize[1])
    }).then(function (result) {
      var crop = result.topCrop;

      _jimp["default"].read(file).then(function (image) {
        image.crop(crop.x, crop.y, crop.width, crop.height).write(newFile);
      });
    })["catch"](function (err) {
      console.error(err);
    });

    resolve();
  });
  return p;
} // export function smartCropAndSaveFile(imageSize, file, newFile) {
//   var cmd = `node node_modules/smartcrop-cli/smartcrop-cli.js --width ${parseInt(
//     imageSize[0]
//   )} --height ${parseInt(imageSize[1])} ${file} ${newFile}`
//   var p = execPromise.exec(cmd)
//   return p
// }


function cropAndSaveFiles(images, file, resp) {
  var length = images.length;
  var cropedImage = 0;
  resp.thumbs = [];
  var p = new _bluebird.Promise(function (resolve) {
    var _loop = function _loop() {
      var image = images[i];

      var ext = _path["default"].extname(file);

      var newFile = file.replace(ext, "_".concat(images[i]).concat(ext));
      resp.thumbs.push({
        name: newFile.replace(_.Manager.instance.pathPublish, ''),
        size: image
      });
      var splitedImage = image.split('x');
      var newWidth = null;
      var newHeight = null;
      if (splitedImage[0] != null && splitedImage[0] != '') newWidth = parseInt(image.split('x')[0]);
      if (splitedImage[1] != null && splitedImage[1] != '') newHeight = parseInt(image.split('x')[1]);

      _jimp["default"].read(file).then(function (originalImage) {
        var originalWidth = originalImage.bitmap.width;
        var originalHeight = originalImage.bitmap.height;
        var ratio = originalWidth * newHeight / newWidth;

        if (newWidth === null || newHeight === null) {
          originalImage.resize(newWidth != null ? newWidth : _jimp["default"].AUTO, newHeight != null ? newHeight : _jimp["default"].AUTO).write(newFile);

          if (++cropedImage === length) {
            resolve(resp);
          }
        } else if (parseInt(ratio - 1) <= parseInt(originalHeight) && parseInt(ratio + 1) >= parseInt(originalHeight)) {
          originalImage.resize(parseInt(image.split('x')[0]), parseInt(image.split('x')[1])).write(newFile);

          if (++cropedImage === length) {
            resolve(resp);
          }
        } else {
          cropAndSaveFile(image.split('x'), file, newFile).then(function (result) {
            if (++cropedImage === length) {
              resolve(resp);
            }
          })["catch"](function (err) {
            console.log(err);
          });
        }
      })["catch"](function (err) {
        console.log(err);
      });
    };

    for (var i = 0; i < length; i++) {
      _loop();
    }
  });
  return p;
}

var generateThumbnail = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(file) {
    var ext, thumbFileName, thumbFileNameRelative, p;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            ext = _path["default"].extname(file).toLowerCase();
            thumbFileName = file.replace(ext, "_thumb".concat(ext));
            thumbFileNameRelative = thumbFileName.replace(_.Manager.instance.pathPublish, '');
            p = new _bluebird.Promise(function (resolve) {
              var cropThumb = cropAndSaveFile([250, 250], file, thumbFileName);
              cropThumb.then(function (result) {
                resolve({
                  thumb: thumbFileNameRelative
                });
              });
            });
            return _context.abrupt("return", p);

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function generateThumbnail(_x) {
    return _ref.apply(this, arguments);
  };
}();

exports.generateThumbnail = generateThumbnail;

function saveFile(req) {
  var p = new _bluebird.Promise(function (resolve) {
    var keepName = req.query.keepName === 'true';
    var resp = {
      success: 1
    };
    var filePath;
    req.pipe(req.busboy);
    req.busboy.on('file', function (fieldname, file, filename, encoding, mimetype) {
      var ext = _path["default"].extname(filename).toLowerCase();

      var slug = createMediaSlug(filename, ext, keepName);
      var mediaType = getMediaType(ext);
      var folderFilePath = createMediaFolder(mediaType);
      var hasSentHeader = false;
      var folderWebPath = '/' + _.config.upload.image;

      if (_.config.upload[mediaType] != null) {
        folderWebPath = '/' + _.config.upload[mediaType];
      }

      filePath = _path["default"].posix.join(folderFilePath, slug);
      resp['filePath'] = _path["default"].posix.join('/' + folderWebPath, slug);
      file.on('limit', function () {
        hasSentHeader = true;
        file.resume();
        resolve({
          error: 1,
          response: 'file is too big'
        });
        return;
      });
      var isValid = isValidMedia(mimetype, ext);

      if (isValid.error) {
        hasSentHeader = true;
        file.resume();
        resolve({
          error: 1,
          response: isValid.error
        });
        return;
      }

      var fstream = _fs["default"].createWriteStream(filePath);

      fstream.on('finish', /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2() {
        var thumbResp, thumbsSizes;
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!hasSentHeader) {
                  _context2.next = 2;
                  break;
                }

                return _context2.abrupt("return");

              case 2:
                if (!(mediaType === 'image')) {
                  _context2.next = 11;
                  break;
                }

                _context2.next = 5;
                return generateThumbnail(filePath);

              case 5:
                thumbResp = _context2.sent;
                resp.thumbnail = /^win/.test(process.platform) ? thumbResp.thumb.replace(/\\/g, '/') : thumbResp.thumb;
                resp = _.abeExtend.hooks.instance.trigger('afterSaveImage', resp, req);

                if (req && req.query && req.query.input && req.query.input.indexOf('data-size') > -1) {
                  thumbsSizes = _.cmsData.regex.getAttr(req.query.input, 'data-size').replace(' ', '').split(',');
                  cropAndSaveFiles(thumbsSizes, filePath, resp).then(function (resp) {
                    if (/^win/.test(process.platform)) {
                      for (var i = 0; i < resp.thumbs.length; i++) {
                        resp.thumbs[i].name = resp.thumbs[i].name.replace(/\\/g, '/');
                      }
                    }

                    resolve(resp);
                  });
                } else resolve(resp); //})


                _context2.next = 12;
                break;

              case 11:
                resolve(resp);

              case 12:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })));
      file.pipe(fstream);
    });
  });
  return p;
}

function isValidMedia(mimetype, ext) {
  var allowedExtensions = _.config.upload.extensions;
  var allowedMimetypes = _.config.upload.mimetypes;
  var error = false;
  if (allowedMimetypes.indexOf(mimetype) < 0) error = ext + ' is not an authorized mimetype';else if (allowedExtensions.indexOf(ext) < 0) error = ext + ' is not an authorized extension';
  return {
    error: error
  };
}

function getMediaType(ext) {
  var type = 'document';

  if (/\.(jpg|jpeg|png|gif|svg|webp)/.test(ext)) {
    type = 'image';
  } else if (/\.(mov|avi|mp4)/.test(ext)) {
    type = 'video';
  } else if (/\.(mp3|wav)/.test(ext)) {
    type = 'sound';
  }

  return type;
}

function createMediaSlug(filename, ext) {
  var keepName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  if (!keepName) {
    var filenameNoExt = _path["default"].basename(filename, ext).toLowerCase();

    return "".concat((0, _slugify["default"])(filenameNoExt, {
      remove: /[$*+~.()'"!\:@§^,;]/g
    }), "-").concat(_.coreUtils.random.generateUniqueIdentifier(2)).concat(ext);
  }

  return filename;
}

function createMediaFolder(mediaType) {
  var folderWebPath = '/' + _.config.upload.image;

  if (_.config.upload[mediaType] != null) {
    folderWebPath = '/' + _.config.upload[mediaType];
  }

  folderWebPath = _.abeExtend.hooks.instance.trigger('beforeSaveImage', folderWebPath);

  var folderFilePath = _path["default"].join(_.Manager.instance.pathPublish, folderWebPath);

  _mkdirp["default"].sync(folderFilePath);

  return folderFilePath;
}

function getThumbsList() {
  var thumbsList = [];

  var pathToThumbs = _path["default"].join(_.Manager.instance.pathPublish, _.config.upload.image);

  var files = _.coreUtils.file.getFilesSync(pathToThumbs, true);

  Array.prototype.forEach.call(files, function (pathFile) {
    pathFile = pathFile.replace(_.Manager.instance.pathPublish, '');

    if (pathFile.indexOf('_thumb.') > -1) {
      thumbsList.push({
        originalFile: pathFile.replace('_thumb.', '.'),
        thumbFile: pathFile
      });
    }
  });
  return thumbsList;
}

function getAssociatedImageFileFromThumb(name) {
  var rexMatchImageName = /_(thumb|\d+x\d+)\./;
  name = _path["default"].join(_path["default"].sep, name);

  var originalName = _path["default"].join(_path["default"].sep, name.replace(rexMatchImageName, '.'));

  var imageList = {
    thumbFile: name,
    originalFile: originalName,
    thumbs: []
  };
  var pathThumb = name.split('/');
  pathThumb.pop();
  pathThumb = _path["default"].join(_.Manager.instance.pathPublish, pathThumb.join('/'));

  var files = _.coreUtils.file.getFilesSync(pathThumb, true);

  Array.prototype.forEach.call(files, function (pathFile) {
    pathFile = pathFile.replace(_.Manager.instance.pathPublish, '');

    if (pathFile !== originalName && pathFile !== name && pathFile.replace(rexMatchImageName, '.') === originalName) {
      imageList.thumbs.push(pathFile);
    }
  });
  return imageList;
}