"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.copy = copy;

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _fsCompare = _interopRequireDefault(require("fs-compare"));

var _dirCompare = require("dir-compare");

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _path = _interopRequireDefault(require("path"));

var _ = require("../../");

function copy() {
  var dest = _.Manager.instance.pathPublish;
  var directory = null;
  var source = _.Manager.instance.pathAssets;

  try {
    directory = _fsExtra["default"].lstatSync(dest);

    if (!directory.isDirectory() && !directory.isSymbolicLink()) {
      _mkdirp["default"].sync(dest);
    }
  } catch (e) {
    _mkdirp["default"].sync(dest);
  }

  try {
    var dirSource = _fsExtra["default"].lstatSync(source);

    if (!dirSource.isDirectory() && !dirSource.isSymbolicLink()) {
      source = null;
    }
  } catch (e) {
    source = null;
  }

  directory = _fsExtra["default"].lstatSync(dest);
  if (directory.isSymbolicLink()) dest = _fsExtra["default"].readlinkSync(dest);

  if (source != null) {
    var res = (0, _dirCompare.compareSync)(source, dest, {
      compareDate: true
    });
    res.diffSet.forEach(function (entry) {
      var state = {
        equal: '==',
        left: '->',
        right: '<-',
        distinct: '<>'
      }[entry.state];
      var name1 = entry.name1 ? entry.name1 : '';
      var name2 = entry.name2 ? entry.name2 : '';
      var exclude = new RegExp(".".concat(_.config.files.templates.extension, "$"));

      if (!exclude.test(name1) && !exclude.test(name2) && entry.type1 !== 'directory' && entry.type2 !== 'directory') {
        if (typeof entry.path1 !== 'undefined' && entry.path1 !== null) {
          var original = entry.path1;
          var basePath = original.replace(_.Manager.instance.pathAssets, '');

          var move = _path["default"].join(dest, basePath);

          if (move === dest) {
            _fsExtra["default"].readdir(original, function (res, items) {
              Array.prototype.forEach.call(items, function (item) {
                var originalItem = _path["default"].join(original, item);

                var lstat = _fsExtra["default"].lstatSync(originalItem);

                var destItem = _path["default"].join(dest, item);

                if (!lstat.isDirectory() && !exclude.test(originalItem)) {
                  try {
                    (0, _fsCompare["default"])(modifiedTime, originalItem, destItem, function (err, diff) {
                      if (diff > 0) _fsExtra["default"].copySync(originalItem, destItem);
                    });
                  } catch (e) {
                    _fsExtra["default"].copySync(originalItem, destItem);
                  }
                }
              });
            });
          } else if (entry.type2 === 'missing' || entry.state === 'distinct') {
            _fsExtra["default"].removeSync(move);

            _fsExtra["default"].copySync(original, move);
          }
        }
      }
    });
  }

  return true;
}

function modifiedTime(fileName, cb) {
  _fsExtra["default"].stat(fileName, function (err, stat) {
    if (err) {
      return cb(err);
    }

    return cb(null, stat.mtime);
  });
}