"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deleteTheme = deleteTheme;
exports.downloadTheme = downloadTheme;
exports.getThemeInfos = getThemeInfos;

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _request = _interopRequireDefault(require("request"));

var _extractZip = _interopRequireDefault(require("extract-zip"));

var _ = require("../../");

function getThemeInfos() {
  var pathToTheme = _path["default"].join(_.config.root, _.config.themes.path, _.config.themes.name);

  var p = new Promise(function (resolve) {
    _fsExtra["default"].readJson(_path["default"].join(pathToTheme, 'theme.json'), function (err, json) {
      // if (err) console.error(err)
      var json = json || {
        theme: {}
      };
      resolve(json.theme);
    });
  })["catch"](function (e) {
    console.error(e);
  });
  return p;
}

function deleteTheme() {
  var pathToTheme = _path["default"].join(_.config.root, _.config.themes.path, _.config.themes.name);

  _fsExtra["default"].readJson(_path["default"].join(pathToTheme, 'theme.json'), function (err, json) {
    if (err) console.error(err);

    if (json != null && json.theme != null && json.theme.root_files != null) {
      Array.prototype.forEach.call(json.theme.root_files, function (file) {
        _fsExtra["default"].remove(_path["default"].join(pathToTheme, file), function (err) {
          if (err) console.error(err);
        });
      });
    }
  });
}

function downloadTheme(url, name) {
  var pathToThemes = _path["default"].join(_.config.root, _.config.themes.path);

  var PathToTmpTheme = _path["default"].join(pathToThemes, 'tmp-download');

  _mkdirp["default"].sync(PathToTmpTheme);

  var p = new Promise(function (resolve) {
    var pathToZip = _path["default"].join(PathToTmpTheme, name + '.zip');

    var writeStream = _fsExtra["default"].createWriteStream(pathToZip);

    (0, _request["default"])(url).on('response', function (res) {}).on('error', function (res) {
      resolve({
        res: 'ko',
        error: res
      });
    }).on('end', function (res) {}).pipe(writeStream);
    writeStream.on('finish', function () {
      (0, _extractZip["default"])(pathToZip, {
        dir: PathToTmpTheme
      }, function (err) {
        var dirs = _.coreUtils.file.getFoldersSync(PathToTmpTheme, false);

        var currentPathToTheme = dirs[0].path;

        _fsExtra["default"].removeSync(_path["default"].join(pathToThemes, name), function (err) {
          if (err) return console.error(err);
        });

        _fsExtra["default"].renameSync(currentPathToTheme, _path["default"].join(pathToThemes, name));

        _fsExtra["default"].remove(PathToTmpTheme, function (err) {
          if (err) return console.error(err);
        });

        if (err != null) {
          console.log(err);
          resolve({
            res: 'ko',
            error: 'err'
          });
          return;
        }

        var json = _.config.getLocalConfig();

        json.themes = {
          name: name
        };

        _.config.save(json);

        _.cmsTemplates.assets.copy();

        getThemeInfos().then(function (json) {
          resolve({
            success: 'ok',
            theme: json
          });
        });
      });
    });
  })["catch"](function (e) {
    resolve({
      res: 'ko',
      error: e
    });
    console.error(e);
  });
  return p;
}