"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _cliColor = _interopRequireDefault(require("cli-color"));

var _extend = _interopRequireDefault(require("extend"));

var _path = _interopRequireDefault(require("path"));

var _config = _interopRequireDefault(require("./config.json"));

var result = _config["default"];
result.root = process.cwd();

if (process.env.ROOT) {
  result.root = process.env.ROOT.replace(/\/$/, '');
}

var hintAbeJson = false;

var loadLocalConfig = function loadLocalConfig(result) {
  var abeConfigPath = _path["default"].join(result.root, 'abe.json');

  if (result.root !== '') {
    try {
      var stat = _fsExtra["default"].statSync(result.root);

      if (stat && stat.isDirectory()) {
        try {
          stat = _fsExtra["default"].statSync(abeConfigPath);

          if (stat) {
            var json = _fsExtra["default"].readJsonSync(abeConfigPath);

            result = (0, _extend["default"])(true, result, json);
          }
        } catch (e) {
          if (!hintAbeJson) {
            hintAbeJson = true; // console.log(
            //   clc.green('[ Hint ]'),
            //   'you can create a specific config file named abe.json to customize your abe install',
            //   clc.cyan.underline('https://github.com/abecms/abecms/blob/master/docs/abe-config.md')
            // )
          }
        }
      }
    } catch (e) {
      console.log('LoadConfig Error');
      console.log(e);
    }
  }
};

loadLocalConfig(result);

result.exist = function (conf, json) {
  var c = conf.split('.');
  var current = json;

  if (typeof current !== 'undefined' && current !== null) {
    Array.prototype.forEach.call(c, function (c) {
      if (current !== false && typeof current[c] !== 'undefined' && current[c] !== null) {
        current = current[c];
      } else {
        current = false;
        return false;
      }
    });
    return current;
  } else {
    return false;
  }
};

result.localConfigExist = function () {
  if (result.root !== '') {
    try {
      var stat = _fsExtra["default"].statSync(result.root);

      if (stat && stat.isDirectory()) {
        try {
          stat = _fsExtra["default"].statSync(_path["default"].join(result.root, 'abe.json'));

          if (stat) {
            return true;
          }
        } catch (e) {
          return false;
        }
      }
    } catch (e) {
      return false;
    }
  }

  return false;
};

result.getLocalConfig = function () {
  if (result.localConfigExist()) {
    return _fsExtra["default"].readJsonSync(_path["default"].join(result.root, 'abe.json'));
  } else {
    return {};
  }
};

result.getDefault = function (conf) {
  return result[conf];
};

result.get = function (conf) {
  return result.exist(conf, result);
};

result.set = function (json) {
  (0, _extend["default"])(true, result, json);
  loadLocalConfig(result);
};

result.extend = function (json) {
  var localConf = result.getLocalConfig();
  (0, _extend["default"])(true, localConf, json);
  result.save(localConf);
};

result.save = function (json) {
  var confPath = _path["default"].join(result.root, 'abe.json');

  _fsExtra["default"].writeJsonSync(confPath, json, {
    space: 2,
    encoding: 'utf-8'
  });

  loadLocalConfig(result);
};

result.getConfigByWebsite = function () {
  var defaultConfig = (0, _extend["default"])(true, {}, result);
  var configBySite = {
    "default": {}
  };
  var localConfig = (0, _extend["default"])(true, {}, defaultConfig);

  for (var item in localConfig) {
    switch (item) {
      case 'intlData':
        configBySite["default"].intlData = localConfig[item];
        break;

      case 'templates':
        configBySite["default"].templates = localConfig[item];
        break;

      case 'structure':
        configBySite["default"].structure = localConfig[item];
        break;

      case 'data':
        configBySite["default"].data = localConfig[item];
        break;

      case 'publish':
        configBySite["default"].publish = localConfig[item];
        break;

      case 'files':
        configBySite["default"].files = {
          templates: {
            extension: localConfig[item].templates.extension
          }
        };
        break;

      case 'upload':
        configBySite["default"].upload = localConfig[item];
        break;
    }
  }

  return configBySite;
};

var _default = result;
exports["default"] = _default;