"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.connectToServer = connectToServer;
exports.exist = exist;
exports.getAllWithKeys = getAllWithKeys;
exports.getDb = getDb;
exports.getDoc = getDoc;
exports.getFileObject = getFileObject;
exports.removeRevision = removeRevision;
exports.saveJson = saveJson;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _2 = require("../../");

var _path = _interopRequireDefault(require("path"));

var _mongodb = require("mongodb");

var _lodash = _interopRequireDefault(require("lodash"));

var options = {
  useNewUrlParser: true,
  poolSize: 10
};
var url = "".concat(_lodash["default"].get(_2.config, "database.mongo.server", "mongodb://localhost"), ":").concat(_lodash["default"].get(_2.config, "database.mongo.port", 27017), "/").concat(_lodash["default"].get(_2.config, "database.mongo.database", "abe"));

var _db;

function connectToServer(callback) {
  if (!url) {
    return callback("Missing mongo URL");
  }

  _mongodb.MongoClient.connect(url, options, function (err, client) {
    if (!client || err) {
      return callback(err);
    }

    _db = client.db(_lodash["default"].get(_2.config, "database.mongo.database", "abe"));
    return callback(err);
  });
}

function getDb() {
  return _db;
}

function getDoc(_x) {
  return _getDoc.apply(this, arguments);
}

function _getDoc() {
  _getDoc = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(jsonPath) {
    var db, JSONs, json, doc;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            jsonPath = _2.cmsData.utils.getRevisionRelativePath(jsonPath);
            db = _2.mongo.getDb();
            JSONs = db.collection("jsons");
            json = {};
            _context.prev = 4;
            _context.next = 7;
            return JSONs.findOne({
              jsonPath: jsonPath
            });

          case 7:
            doc = _context.sent;

            if (doc) {
              json = doc.json;
              json.path = jsonPath;
            } else {
              json = {
                err: true
              };
            }

            _context.next = 14;
            break;

          case 11:
            _context.prev = 11;
            _context.t0 = _context["catch"](4);
            console.error(_context.t0);

          case 14:
            return _context.abrupt("return", json);

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[4, 11]]);
  }));
  return _getDoc.apply(this, arguments);
}

function getAllWithKeys(_x2) {
  return _getAllWithKeys.apply(this, arguments);
}

function _getAllWithKeys() {
  _getAllWithKeys = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(withKeys) {
    var db, JSONs, docs, docsArr, i, fjson, fileObject;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            db = _2.mongo.getDb();
            JSONs = db.collection("jsons");
            _context2.next = 4;
            return JSONs.find().toArray();

          case 4:
            docs = _context2.sent;
            docsArr = [];

            for (i in docs) {
              fjson = docs[i].json;
              fileObject = {
                date: "" + docs[i].mtime.toString() + "",
                path: docs[i].jsonPath
              };
              fileObject = _2.cmsData.file.getAbeMeta(fileObject, fjson);
              docsArr.push(fileObject);
            }

            return _context2.abrupt("return", docsArr);

          case 8:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _getAllWithKeys.apply(this, arguments);
}

function exist(_x3) {
  return _exist.apply(this, arguments);
}

function _exist() {
  _exist = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(jsonPath) {
    var exists, db, JSONs, doc;
    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            jsonPath = _2.cmsData.utils.getRevisionRelativePath(jsonPath);
            exists = false;
            db = _2.mongo.getDb();
            JSONs = db.collection("jsons");
            _context3.prev = 4;
            _context3.next = 7;
            return JSONs.findOne({
              jsonPath: jsonPath
            });

          case 7:
            doc = _context3.sent;

            if (doc) {
              exists = true;
            }

            _context3.next = 14;
            break;

          case 11:
            _context3.prev = 11;
            _context3.t0 = _context3["catch"](4);
            console.error(_context3.t0);

          case 14:
            return _context3.abrupt("return", exists);

          case 15:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, null, [[4, 11]]);
  }));
  return _exist.apply(this, arguments);
}

function getFileObject(json) {
  var name = _path["default"].basename(json.path);

  name = _2.cmsData.fileAttr["delete"](name);
  var date = json.updatedDate;
  var fileObject = {
    name: name,
    path: json.path,
    date: date
  };
  return fileObject;
}

function removeRevision(_x4) {
  return _removeRevision.apply(this, arguments);
}

function _removeRevision() {
  _removeRevision = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(jsonPath) {
    var db, JSONs;
    return _regenerator["default"].wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            db = _2.mongo.getDb();
            JSONs = db.collection("jsons");
            _context4.prev = 2;
            _context4.next = 5;
            return JSONs.deleteOne({
              jsonPath: jsonPath
            });

          case 5:
            _context4.next = 11;
            break;

          case 7:
            _context4.prev = 7;
            _context4.t0 = _context4["catch"](2);
            console.error(_context4.t0);
            return _context4.abrupt("return", false);

          case 11:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, null, [[2, 7]]);
  }));
  return _removeRevision.apply(this, arguments);
}

function saveJson(_x5, _x6) {
  return _saveJson.apply(this, arguments);
}

function _saveJson() {
  _saveJson = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(jsonPath, json) {
    var db, JSONs, mtime;
    return _regenerator["default"].wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            db = _2.mongo.getDb();
            JSONs = db.collection("jsons");
            jsonPath = _2.cmsData.utils.getRevisionRelativePath(jsonPath);
            _context5.prev = 3;
            mtime = Date.now();
            _context5.next = 7;
            return JSONs.updateOne({
              jsonPath: jsonPath
            }, {
              $set: {
                json: json,
                jsonPath: jsonPath,
                mtime: mtime
              }
            }, {
              upsert: true
            });

          case 7:
            _context5.next = 13;
            break;

          case 9:
            _context5.prev = 9;
            _context5.t0 = _context5["catch"](3);
            console.error(_context5.t0);
            return _context5.abrupt("return", false);

          case 13:
            return _context5.abrupt("return", true);

          case 14:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5, null, [[3, 9]]);
  }));
  return _saveJson.apply(this, arguments);
}