"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addDateIsoToRevisionPath = addDateIsoToRevisionPath;
exports.addFolder = addFolder;
exports.changePath = changePath;
exports.exist = exist;
exports.getContent = getContent;
exports.getDate = getDate;
exports.getFiles = getFiles;
exports.getFilesSync = getFilesSync;
exports.getFoldersAsync = getFoldersAsync;
exports.getFoldersSync = getFoldersSync;
exports.getJson = getJson;
exports.removeFolder = removeFolder;
exports.renameFolder = renameFolder;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _bluebird = _interopRequireDefault(require("bluebird"));

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _ = require("../../");

function exist(pathFile) {
  try {
    _fsExtra["default"].statSync(pathFile);

    return true;
  } catch (e) {
    return false;
  }
}

function changePath(pathEnv, change) {
  pathEnv = pathEnv.split(_path["default"].sep).join('/').replace(_.config.root, '').replace(/^\//, '').split('/');
  pathEnv[0] = change;
  return _path["default"].join(_.config.root, pathEnv.join('/'));
}

function getJson(pathJson) {
  var json = {};

  try {
    var stat = _fsExtra["default"].statSync(pathJson);

    if (stat) {
      json = _fsExtra["default"].readJsonSync(pathJson);
    }
  } catch (e) {
    json = {};
  }

  return json;
}
/**
 * This method checks that the path leads to a file and return the content as UTF-8 content
 * @param  {string} path The path
 * @return {string}      The content of the UTF-8 file
 */


function getContent(pathFile) {
  var res = null;

  if (typeof pathFile !== 'undefined' && pathFile !== null && pathFile !== '') {
    if (exist(pathFile)) {
      res = _fsExtra["default"].readFileSync(pathFile, 'utf8');
    }
  }

  return res;
}
/**
 * synchronous fse walker to get folders with recursive option
 * @param  {String}  dirname   dir path
 * @param  {Boolean} recursive do we recurse in the subfolders
 * @param  {String}  filterExt extension or ''
 * @return {array}             array of pathfiles
 */


function getFoldersSync(dirname) {
  var recursive = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var items = [];

  try {
    _fsExtra["default"].readdirSync(dirname).map(function (fileName) {
      var pathFile = _path["default"].join(dirname, fileName);

      var stat = _fsExtra["default"].statSync(pathFile);

      if (stat.isDirectory()) {
        var directory = {
          path: pathFile,
          folders: []
        };

        if (recursive) {
          directory.folders = _.coreUtils.file.getFoldersSync(pathFile, recursive);
        }

        items.push(directory);
      }
    });

    return items;
  } catch (e) {
    return items;
  }
}
/**
 * Promisified fse walker to get folders with recursive option
 * @param  {String}  dirname   dir path
 * @param  {Boolean} recursive do we recurse in the subfolders
 * @param  {String}  filterExt extension or ''
 * @return {array}             array of pathfiles
 */


function getFoldersAsync(dirname) {
  var recursive = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var items = [];
  return _fsExtra["default"].readdirAsync(dirname).map(function (fileName) {
    var pathFile = _path["default"].join(dirname, fileName);

    return _fsExtra["default"].statAsync(pathFile).then(function (stat) {
      if (stat.isDirectory()) {
        var directory = {
          path: pathFile,
          folders: []
        };

        if (recursive) {
          return _.coreUtils.file.getFoldersAsync(pathFile, recursive).then(function (filesInDir) {
            directory.folders = filesInDir;
            items.push(directory);
          });
        } else {
          items.push(directory);
        }
      }

      return;
    });
  }).then(function () {
    return items;
  });
}
/**
 * synchronous fse walker with recursive and extension options
 * @param  {String}  dirname   dir path
 * @param  {Boolean} recursive do we recurse in the subfolders
 * @param  {String}  filterExt extension or ''
 * @return {array}             array of pathfiles
 */


function getFilesSync(dirname) {
  var recursive = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var filterExt = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  var items = [];

  try {
    _fsExtra["default"].readdirSync(dirname).map(function (fileName) {
      var pathFile = _path["default"].join(dirname, fileName);

      var stat = _fsExtra["default"].statSync(pathFile);

      if (stat.isFile()) {
        var extFile = _path["default"].extname(fileName);

        if (filterExt === '' || extFile === filterExt) {
          items.push(pathFile);
        }
      }

      if (stat.isDirectory() && recursive) {
        var filesInDir = _.coreUtils.file.getFilesSync(pathFile, recursive, filterExt);

        items = items.concat(filesInDir);
      }
    });

    return items;
  } catch (e) {
    return items;
  }
}
/**
 * Promisified fse walker with recursive and extension options
 * @param  {String}  dirname   dir path
 * @param  {Boolean} recursive do we recurse in the subfolders
 * @param  {String}  filterExt extension or ''
 * @return {array}             array of pathfiles
 */


function getFiles(_x) {
  return _getFiles.apply(this, arguments);
}

function _getFiles() {
  _getFiles = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(dirname) {
    var recursive,
        filterExt,
        items,
        stat,
        files,
        _args2 = arguments;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            recursive = _args2.length > 1 && _args2[1] !== undefined ? _args2[1] : true;
            filterExt = _args2.length > 2 && _args2[2] !== undefined ? _args2[2] : '';
            items = [];
            _context2.prev = 3;
            _context2.next = 6;
            return _fsExtra["default"].lstat(dirname);

          case 6:
            stat = _context2.sent;

            if (!stat.isDirectory()) {
              _context2.next = 13;
              break;
            }

            _context2.next = 10;
            return _fsExtra["default"].readdir(dirname);

          case 10:
            files = _context2.sent;
            _context2.next = 13;
            return _bluebird["default"].all(files.map( /*#__PURE__*/function () {
              var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(fileName) {
                var pathFile, statFile, extFile, filesInDir;
                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        pathFile = _path["default"].join(dirname, fileName);
                        _context.next = 3;
                        return _fsExtra["default"].stat(pathFile);

                      case 3:
                        statFile = _context.sent;

                        if (statFile.isFile()) {
                          extFile = _path["default"].extname(fileName);

                          if (filterExt === '' || extFile === filterExt) {
                            items.push(pathFile);
                          }
                        }

                        if (!recursive) {
                          _context.next = 10;
                          break;
                        }

                        _context.next = 8;
                        return _.coreUtils.file.getFiles(pathFile, recursive, filterExt);

                      case 8:
                        filesInDir = _context.sent;
                        items = items.concat(filesInDir);

                      case 10:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              return function (_x2) {
                return _ref.apply(this, arguments);
              };
            }()));

          case 13:
            _context2.next = 17;
            break;

          case 15:
            _context2.prev = 15;
            _context2.t0 = _context2["catch"](3);

          case 17:
            return _context2.abrupt("return", items);

          case 18:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[3, 15]]);
  }));
  return _getFiles.apply(this, arguments);
}

function addFolder(folderPath) {
  _mkdirp["default"].sync(_path["default"].join(_.config.root, folderPath));

  return folderPath;
}

function renameFolder(oldFolderPath, folderPath) {
  _fsExtra["default"].renameSync(_path["default"].join(_.config.root, oldFolderPath), _path["default"].join(_.config.root, folderPath));

  return folderPath;
}

function removeFolder(folderPath) {
  _fsExtra["default"].removeSync(_path["default"].join(_.config.root, folderPath));

  return folderPath;
}
/**
 * Return the date of the revision
 * @param  {String}  revisionPath   url of the post revision
 * @return {Date}    date           Date object
 */


function getDate(revisionPath) {
  var date = _.cmsData.fileAttr.get(revisionPath).d;

  if (typeof date === 'undefined' || date === null || date === '') {
    date = new Date();
  } else {
    date = new Date(date);
  }

  return date;
}
/**
 *
 * @param  {String}  revisionPath   url of the post revision
 * @return {Date}    date           Date object
 */


function addDateIsoToRevisionPath(revisionPath, type) {
  var dateISO;
  var saveJsonFile = revisionPath;

  if (type === 'publish') {
    return revisionPath;
  }

  dateISO = type[0] + _.cmsData.revision.removeStatusAndDateFromFileName(new Date().toISOString());

  if (dateISO) {
    saveJsonFile = _.cmsData.fileAttr.add(saveJsonFile, dateISO);
  }

  return saveJsonFile;
}