"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _child_process = require("child_process");

var _bluebird = require("bluebird");

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _cliColor = _interopRequireDefault(require("cli-color"));

var _which = _interopRequireDefault(require("which"));

var _extend = _interopRequireDefault(require("extend"));

var _ = require("../");

var npm = _which["default"].sync('npm');

var singleton = Symbol();
var singletonEnforcer = Symbol();

var Plugins = /*#__PURE__*/function () {
  function Plugins(enforcer) {
    var _this = this;

    (0, _classCallCheck2["default"])(this, Plugins);
    if (enforcer != singletonEnforcer) throw 'Cannot construct Plugins singleton';
    this.pluginsDir = _path["default"].join(_.config.root, 'node_modules') + _path["default"].sep;
    this.scriptsDir = _path["default"].join(_.config.root, 'scripts') + _path["default"].sep;
    this._plugins = []; // Plugins

    if (_.config.plugins && Array.isArray(_.config.plugins)) {
      Array.prototype.forEach.call(_.config.plugins, function (pluginEntry) {
        var plugin = _this.getPluginConfig(_this.pluginsDir, pluginEntry);

        _this._plugins.push(plugin);
      });
    } // Scripts


    try {
      var directoryScripts = _fsExtra["default"].lstatSync(this.scriptsDir);

      if (directoryScripts.isDirectory()) {
        this._scripts = _.coreUtils.file.getFoldersSync(this.scriptsDir, false);
        Array.prototype.forEach.call(this._scripts, function (scriptEntry) {
          var name = scriptEntry.path.replace(_this.scriptsDir, '');

          var script = _this.getPluginConfig(_this.scriptsDir, name);

          _this._plugins.push(script);
        });
      }
    } catch (e) {}
  }

  (0, _createClass2["default"])(Plugins, [{
    key: "getPluginConfig",
    value: function getPluginConfig(dir, entry) {
      // remove npm version if any
      var pluginId = entry.split('@')[0]; // remove github version if any

      pluginId = pluginId.split('#')[0]; // remove github path if any

      pluginId = _path["default"].basename(pluginId);

      var plugHook = _path["default"].join(dir, pluginId, _.config.hooks.url, 'hooks.js');

      var plugCustom = _path["default"].join(dir, pluginId, 'custom');

      var plugPartials = _path["default"].join(dir, pluginId, 'partials');

      var plugTemplates = _path["default"].join(dir, pluginId, 'templates');

      var plugProcess = _path["default"].join(dir, pluginId, 'process');

      var plugRoutes = _path["default"].join(dir, pluginId, 'routes');

      var plugin = {
        hooks: null,
        partials: null,
        templates: null,
        process: null,
        routes: null,
        custom: null
      };

      try {
        var fileHook = _fsExtra["default"].lstatSync(plugHook);

        if (fileHook.isFile()) {
          try {
            delete require.cache[require.resolve(plugHook)];

            var h = require(plugHook);
          } catch (e) {
            console.log(e.stack);
            console.log(_cliColor["default"].green('[ Hint ]'), "It seems that you don't have the npm module @babel/preset-env installed on your project");
          }

          plugin.hooks = h["default"];
        }
      } catch (e) {
        plugin.hooks = null;
      }

      try {
        var directoryCustom = _fsExtra["default"].lstatSync(plugCustom);

        if (directoryCustom.isDirectory()) {
          plugin.custom = plugCustom;
        }
      } catch (e) {
        plugin.custom = null;
      }

      try {
        var directoryPartials = _fsExtra["default"].lstatSync(plugPartials);

        if (directoryPartials.isDirectory()) {
          plugin.partials = plugPartials;
        }
      } catch (e) {
        plugin.partials = null;
      }

      try {
        var directoryTemplates = _fsExtra["default"].lstatSync(plugTemplates);

        if (directoryTemplates.isDirectory()) {
          plugin.templates = plugTemplates;
        }
      } catch (e) {
        plugin.templates = null;
      }

      try {
        var directoryProcess = _fsExtra["default"].lstatSync(plugProcess);

        if (directoryProcess.isDirectory()) {
          plugin.process = {};

          var processFiles = _.coreUtils.file.getFilesSync(plugProcess, false);

          Array.prototype.forEach.call(processFiles, function (processFile) {
            plugin.process[_path["default"].basename(processFile, '.js')] = processFile;
          });
        }
      } catch (e) {
        plugin.process = null;
      }

      try {
        var directoryRoute = _fsExtra["default"].lstatSync(plugRoutes);

        if (directoryRoute.isDirectory()) {
          plugin.routes = {};

          var gets = _path["default"].join(plugRoutes, 'get');

          try {
            var directoryGets = _fsExtra["default"].lstatSync(gets);

            if (directoryGets.isDirectory()) {
              var routesGet = [];

              var routePaths = _.coreUtils.file.getFilesSync(gets, false);

              Array.prototype.forEach.call(routePaths, function (route) {
                var pathUrl = "/abe/plugin/".concat(pluginId, "/").concat(_path["default"].basename(route, '.js'), "*");
                var routeObject = {
                  path: route,
                  routePath: pathUrl
                };
                routesGet.push(routeObject);
              });
              plugin.routes.get = routesGet;
            }
          } catch (e) {
            plugin.routes.get = null;
          }

          try {
            var posts = _path["default"].join(plugRoutes, 'post');

            var directoryPosts = _fsExtra["default"].lstatSync(posts);

            if (directoryPosts.isDirectory()) {
              var routesPost = [];

              var _routePaths = _.coreUtils.file.getFilesSync(posts, false);

              Array.prototype.forEach.call(_routePaths, function (route) {
                var pathUrl = "/abe/plugin/".concat(pluginId, "/").concat(_path["default"].basename(route, '.js'), "*");
                var routeObject = {
                  path: route,
                  routePath: pathUrl
                };
                routesPost.push(routeObject);
              });
              plugin.routes.post = routesPost;
            }
          } catch (e) {
            plugin.routes.post = null;
          }
        }
      } catch (e) {
        plugin.routes = null;
      }

      return plugin;
    }
  }, {
    key: "getProcess",
    value: function getProcess(fn) {
      var proc = null;

      if (typeof this._plugins !== 'undefined' && this._plugins !== null) {
        Array.prototype.forEach.call(this._plugins, function (plugin) {
          if (typeof plugin.process !== 'undefined' && plugin.process !== null && typeof plugin.process[fn] !== 'undefined' && plugin.process[fn] !== null) {
            proc = plugin.process[fn];
          }
        });
      }

      return proc;
    }
  }, {
    key: "hooks",
    value: function hooks() {
      var _this2 = this;

      if (arguments.length > 0) {
        var args = [].slice.call(arguments);
        var fn = args.shift();

        if (this._plugins != null) {
          Array.prototype.forEach.call(this._plugins, function (plugin) {
            if (plugin.hooks != null && plugin.hooks[fn] != null) {
              args[0] = plugin.hooks[fn].apply(_this2, args);
            }
          });
        }
      } else {
        args = [''];
      }

      return args[0];
    }
  }, {
    key: "getPartials",
    value: function getPartials() {
      var partials = [];
      Array.prototype.forEach.call(this._plugins, function (plugin) {
        if (typeof plugin.partials !== 'undefined' && plugin.partials !== null) {
          partials.push(plugin.partials);
        }
      });
      return partials;
    }
  }, {
    key: "getCustoms",
    value: function getCustoms() {
      var customs = [];
      Array.prototype.forEach.call(this._plugins, function (plugin) {
        if (typeof plugin.custom !== 'undefined' && plugin.custom !== null) {
          customs.push(plugin.custom);
        }
      });
      return customs;
    }
  }, {
    key: "getRoutes",
    value: function getRoutes() {
      var routes = [];
      Array.prototype.forEach.call(this._plugins, function (plugin) {
        if (typeof plugin.routes !== 'undefined' && plugin.routes !== null) {
          routes = routes.concat(plugin.routes);
        }
      });
      return routes;
    }
  }, {
    key: "removePlugin",
    value: function removePlugin(plugin) {
      var pluginName = plugin.split('@')[0];
      pluginName = pluginName.split('#')[0];

      if (_.config.localConfigExist()) {
        var json = _.config.getLocalConfig();

        if (typeof json.plugins !== 'undefined' && json.plugins !== null) {
          Array.prototype.forEach.call(json.plugins, function (plugged, index) {
            if (pluginName === _path["default"].basename(plugged.split('@')[0].split('#')[0])) {
              json.plugins.splice(index, 1);

              _.config.save(json);
            }
          });
        }
      }
    }
  }, {
    key: "updateScripts",
    value: function updateScripts() {
      var _this3 = this;

      this._plugins.length = 0;
      this._scripts.length = 0; // Plugins

      if (_.config.plugins && Array.isArray(_.config.plugins)) {
        Array.prototype.forEach.call(_.config.plugins, function (pluginEntry) {
          var plugin = _this3.getPluginConfig(_this3.pluginsDir, pluginEntry);

          _this3._plugins.push(plugin);
        });
      } // Scripts


      try {
        var directoryScripts = _fsExtra["default"].lstatSync(this.scriptsDir);

        if (directoryScripts.isDirectory()) {
          this._scripts = _.coreUtils.file.getFoldersSync(this.scriptsDir, false);
          Array.prototype.forEach.call(this._scripts, function (scriptEntry) {
            var name = scriptEntry.path.replace(_this3.scriptsDir, '');

            var script = _this3.getPluginConfig(_this3.scriptsDir, name);

            _this3._plugins.push(script);
          });
        }
      } catch (e) {}
    }
  }, {
    key: "updatePlugin",
    value: function updatePlugin(plugin) {
      var json = {};
      var createLocalConfig = true;
      var pluginName = plugin.split('@')[0];
      pluginName = pluginName.split('#')[0];
      var pluginIdArray = pluginName.split('/');
      var pluginId = pluginIdArray[pluginIdArray.length - 1];

      if (_.config.localConfigExist()) {
        json = _.config.getLocalConfig();
        createLocalConfig = false;
      }

      if (typeof json.plugins === 'undefined' || json.plugins === null) {
        json.plugins = [plugin];
      } else {
        var found = false;
        Array.prototype.forEach.call(json.plugins, function (plugged, index) {
          if (pluginName === plugged.split('@')[0].split('#')[0]) {
            json.plugins[index] = plugin;
            found = true;
          }
        });

        if (!found) {
          json.plugins.push(plugin);
        }
      }

      if (createLocalConfig) {
        console.log(_cliColor["default"].green('[ Hint ]'), 'creating a local config abe.json with your plugin definition', _cliColor["default"].cyan.underline('https://github.com/abecms/abecms/blob/master/docs/abe-config.md'));
      }

      try {
        var stat = _fsExtra["default"].lstatSync(_path["default"].join(this.pluginsDir, pluginId, 'abe.json'));

        if (stat.isFile()) {
          var pluginDefaultConfig = _fsExtra["default"].readJsonSync(_path["default"].join(this.pluginsDir, pluginId, 'abe.json'));

          (0, _extend["default"])(true, pluginDefaultConfig, json);
          json = pluginDefaultConfig;
        }
      } catch (e) {}

      _.config.save(json);
    }
  }, {
    key: "uninstall",
    value: function uninstall(dir, plugin) {
      if (plugin !== null) {
        this.remove(dir, plugin).then(function () {
          return 0;
        });
      } else {
        console.log(_cliColor["default"].cyan('no plugin with this name found'));
        return 0;
      }
    }
  }, {
    key: "install",
    value: function install(dir) {
      var _this4 = this;

      var plugin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      if (plugin !== null) {
        this.add(dir, plugin).then(function () {
          return 0;
        });
      } else {
        if (_.config.plugins && Array.isArray(_.config.plugins)) {
          var ps = [];
          Array.prototype.forEach.call(_.config.plugins, function (plugin) {
            ps.push(_this4.add(dir, plugin));
          });

          _bluebird.Promise.all(ps).then(function () {
            return 0;
          });
        } else {
          console.log(_cliColor["default"].cyan('no plugin found'));
          return 0;
        }
      }
    }
  }, {
    key: "remove",
    value: function remove(dir, plugin) {
      var _this5 = this;

      var p = new _bluebird.Promise(function (resolve) {
        try {
          console.log(_cliColor["default"].green('spawn'), _cliColor["default"].cyan('npm uninstall ' + plugin)); // const npmUninstall = spawn('npm', ['uninstall', pluginDir]);

          var npmUninstall = (0, _child_process.spawn)(npm, ['uninstall', '--save', plugin], {
            cwd: dir
          });
          npmUninstall.stdout.on('data', function (data) {
            console.log('' + data);
          });
          npmUninstall.stderr.on('data', function (data) {
            console.log('' + data);
          });
          npmUninstall.on('close', function (code) {
            console.log(_cliColor["default"].cyan('child process exited with code'), code);

            if (code == 0) {
              _this5.removePlugin(plugin);
            }

            resolve();
          });
          npmUninstall.on('error', function (err) {
            console.log(_cliColor["default"].red('cannot uninstall npm dependencies for'), dir);
            console.log(err);
            resolve(err);
          });
        } catch (err) {
          console.log(_cliColor["default"].cyan('uninstall'), err);
          resolve(err);
        }
      });
      return p;
    }
  }, {
    key: "add",
    value: function add(dir, plugin) {
      var _this6 = this;

      var p = new _bluebird.Promise(function (resolve) {
        try {
          console.log(_cliColor["default"].green('spawn'), _cliColor["default"].cyan('npm install ' + plugin));
          var npmPackage = (0, _child_process.spawn)(npm, ['init', '--force', {
            cwd: dir
          }]);
          npmPackage.on('close', function (code) {
            var npmInstall = (0, _child_process.spawn)(npm, ['install', '--save --prefix ' + dir, plugin]);
            npmInstall.stdout.on('data', function (data) {
              console.log('' + data);
            });
            npmInstall.stderr.on('data', function (data) {
              console.log('' + data);
            });
            npmInstall.on('close', function (code) {
              console.log(_cliColor["default"].cyan('child process exited with code'), code);

              if (code == 0) {
                _this6.updatePlugin(plugin);
              }

              resolve();
            });
            npmInstall.on('error', function (err) {
              console.log(_cliColor["default"].red('cannot install npm dependencies for'), dir);
              console.log(err);
              resolve(err);
            });
          });
          npmPackage.on('error', function (err) {
            console.log(_cliColor["default"].red('cannot create the package.json file'), dir);
            console.log(err);
            resolve(err);
          });
        } catch (err) {
          console.log(_cliColor["default"].cyan('install'), err);
          resolve(err);
        }
      });
      return p;
    }
  }], [{
    key: "instance",
    get: function get() {
      if (!this[singleton]) {
        this[singleton] = new Plugins(singletonEnforcer);
      }

      return this[singleton];
    }
  }]);
  return Plugins;
}();

var _default = Plugins;
exports["default"] = _default;