"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addActivity = addActivity;
exports.checkSameEmail = checkSameEmail;
exports.commonPassword = commonPassword;
exports.decodeUser = decodeUser;
exports.encryptPassword = encryptPassword;
exports.find = find;
exports.findByEmail = findByEmail;
exports.findByResetPasswordToken = findByResetPasswordToken;
exports.findByUsername = findByUsername;
exports.findSync = findSync;
exports.getActivity = getActivity;
exports.getAll = getAll;
exports.getGravatarImage = getGravatarImage;
exports.getRole = getRole;
exports.getTokenFromAuthHeader = getTokenFromAuthHeader;
exports.getTokenFromCookies = getTokenFromCookies;
exports.getTokenFromQuery = getTokenFromQuery;
exports.getUserRoutes = getUserRoutes;
exports.getUserWorkflow = getUserWorkflow;
exports.isAbeRestrictedUrl = isAbeRestrictedUrl;
exports.isUserAllowedOnRoute = isUserAllowedOnRoute;
exports.isValid = isValid;
exports.loginLimitTry = loginLimitTry;

var _redis = _interopRequireDefault(require("redis"));

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _ratelimiter = _interopRequireDefault(require("ratelimiter"));

var _owaspPasswordStrengthTest = _interopRequireDefault(require("owasp-password-strength-test"));

var _bcryptNodejs = _interopRequireDefault(require("bcrypt-nodejs"));

var _cookies = _interopRequireDefault(require("cookies"));

var _jwtSimple = _interopRequireDefault(require("jwt-simple"));

var _cli = require("../../cli");

function checkSameEmail(data) {
  var emailAlreadyUsed = false;

  var bdd = _cli.User.manager.instance.get();

  for (var i = 0, len = bdd.length; i < len; i++) {
    var user = bdd[i];

    if (user.email === data.email && parseInt(user.id) !== parseInt(data.id)) {
      emailAlreadyUsed = true;
    }
  }

  if (emailAlreadyUsed === true) {
    return {
      success: 0,
      message: 'Email adress already exist'
    };
  } else {
    return {
      success: 1
    };
  }
}

function getRole(data) {
  var roles = _cli.config.users.roles;
  Array.prototype.forEach.call(roles, function (role) {
    if (role.name === data.role) {
      data.role = role;
    }
  });
  return data;
}

function commonPassword(data) {
  var owaspConfig = _cli.config.users.owasp;

  _owaspPasswordStrengthTest["default"].config(owaspConfig);

  var sameAsUser = typeof owaspConfig.sameAsUser !== 'undefined' && owaspConfig.sameAsUser !== null ? owaspConfig.sameAsUser : true;
  var mostCommon = typeof owaspConfig.mostCommon !== 'undefined' && owaspConfig.mostCommon !== null ? owaspConfig.mostCommon : true;
  var mostCommonPassword = _cli.config.users.mostCommonPassword;

  _owaspPasswordStrengthTest["default"].tests.required.push(function (password) {
    if (mostCommon && _cli.coreUtils.array.contains(mostCommonPassword, password.toLowerCase())) {
      return 'the password used is too common.';
    }
  });

  var currentUserName = data.username;

  _owaspPasswordStrengthTest["default"].tests.required.push(function (password) {
    var username = currentUserName;
    var shouldTest = sameAsUser;

    if (shouldTest) {
      if (password.toLowerCase() === username.toLowerCase()) {
        return 'username and password must be different.';
      }

      if (password.toLowerCase() === username.toLowerCase().split('').reverse().join('')) {
        return 'username and password must be different, not just inverted.';
      }
    }
  });

  var res = _owaspPasswordStrengthTest["default"].test(data.password);

  if (typeof res.errors !== 'undefined' && res.errors !== null && res.errors.length > 0) {
    var message = '';
    Array.prototype.forEach.call(res.errors, function (error) {
      message += error + '<br />';
    });
    return {
      success: 0,
      message: message
    };
  } else {
    return {
      success: 1
    };
  }
}

function encryptPassword(numb, password) {
  var salt = _bcryptNodejs["default"].genSaltSync(numb);

  return _bcryptNodejs["default"].hashSync(password, salt);
}

function getUserRoutes(workflow) {
  var routes = _cli.config.users.routes;
  var userRoles = [];
  Array.prototype.forEach.call(Object.keys(routes), function (role) {
    if (role === workflow) {
      userRoles = routes[role];
    }
  });
  return userRoles;
}

function findSync(id) {
  var bdd = _cli.User.manager.instance.get();

  for (var i = 0, len = bdd.length; i < len; i++) {
    var user = bdd[i];

    if (parseInt(user.id) === parseInt(id)) {
      return user;
    }
  }

  return null;
}

function find(id, done) {
  var bdd = _cli.User.manager.instance.get();

  for (var i = 0, len = bdd.length; i < len; i++) {
    var user = bdd[i];

    if (parseInt(user.id) === parseInt(id)) {
      return done(null, user);
    }
  }

  return done(null, null);
}

function findByUsername(username, done) {
  var bdd = _cli.User.manager.instance.get();

  for (var i = 0, len = bdd.length; i < len; i++) {
    var user = bdd[i];

    if (user.username === username) {
      return done(null, user);
    }
  }

  return done(null, null);
}

function findByEmail(email, done) {
  var user = _cli.User.manager.instance.findByEmail(email);

  if (user) {
    return done(null, user);
  }

  return done(null, null);
}

function findByResetPasswordToken(resetPasswordToken, done) {
  var user = _cli.User.manager.instance.resetPasswordToken(resetPasswordToken);

  if (user) {
    return done(null, user);
  }

  return done(null, null);
}

function isValid(user, password) {
  if (user.actif === 1) {
    if (_bcryptNodejs["default"].compareSync(password, user.password)) {
      return true;
    }
  }

  return false;
}

function decodeUser(req, res) {
  var decoded = {};

  var token = _cli.User.utils.getTokenFromQuery(req, res) || _cli.User.utils.getTokenFromCookies(req, res) || _cli.User.utils.getTokenFromAuthHeader(req, res);

  if (typeof token !== 'undefined' && token !== null && token !== '') {
    try {
      var secret = _cli.config.users.secret;
      decoded = _jwtSimple["default"].decode(token, secret);
    } catch (err) {}
  }

  return decoded;
}

function getTokenFromQuery(req, res) {
  if (req.body && req.body.token) {
    return req.body.token;
  } else if (req.query && req.query.token) {
    return req.query.token;
  } else if (req.headers && req.headers['x-access-token']) {
    return req.headers['x-access-token'];
  }
}

function getTokenFromAuthHeader(req, res) {
  if (req.headers && req.headers.authorization && req.headers.authorization.split(' ')[0] === 'Bearer') {
    return req.headers.authorization.split(' ')[1];
  }
}

function getTokenFromCookies(req, res) {
  var cookies = new _cookies["default"](req, res, {
    secure: _cli.config.cookie.secure
  });
  return cookies.get('x-access-token');
} // the last test !/^\/abe\//.test(currentRoute + '/') has a "+ '/'" to take /abe route into account


function isAbeRestrictedUrl(currentRoute) {
  if (currentRoute.indexOf('/abe/users/forgot') > -1 || currentRoute.indexOf('/abe/users/login') > -1 || currentRoute.indexOf('/abe/users/reset') > -1 || currentRoute.indexOf('/abe/rest/') > -1 || !/^\/abe\//.test(currentRoute + '/')) {
    return false;
  }

  return true;
}

function isUserAllowedOnRoute(workflow, currentRoute) {
  var isAllowed = false;

  if (_cli.config.users.enable) {
    if (currentRoute.indexOf('/abe/users/forgot') > -1 || currentRoute.indexOf('/abe/users/login') > -1 || !/^\/abe/.test(currentRoute)) {
      return true;
    }

    if (currentRoute.indexOf('/abe') === -1) {
      isAllowed = true;
    }

    if (workflow != null) {
      var routes = _cli.config.users.routes;

      if (typeof routes[workflow] !== 'undefined' && routes[workflow] !== null) {
        Array.prototype.forEach.call(routes[workflow], function (route) {
          var reg = new RegExp(route);

          if (reg.test(currentRoute)) {
            isAllowed = true;
          }
        });
      }
    }
  } else {
    isAllowed = true;
  }

  return isAllowed;
}

function getAll() {
  return _cli.User.manager.instance.get();
}

function getUserWorkflow(status) {
  var flows = [];

  function addFlow(flow, type, action) {
    type = type != null ? type : flow;
    return {
      status: flow,
      url: "/abe/operations/".concat(action, "/").concat(type)
    };
  }

  if (_cli.config.users.enable) {
    var found = null;
    Array.prototype.forEach.call(_cli.config.users.workflow, function (flow) {
      if (found != null) {
        flows.push(addFlow(flow, flow, 'submit'));
        found = null;
      }

      if (status == flow) {
        found = flow;

        if (flow != 'draft' && flow != 'publish') {
          flows.push(addFlow('reject', flow, 'reject'));
        }

        if (flow == 'publish') {
          flows.push(addFlow('edit', 'draft', 'edit'));
        } else {
          flows.push(addFlow('save', flow, 'edit'));
        }
      }
    });

    if (found != null) {
      flows.push(addFlow('save', 'publish', 'submit'));
    }
  } else {
    flows = [addFlow('draft', 'draft', 'submit'), addFlow('publish', 'publish', 'submit')];
  }

  return flows;
}

function loginLimitTry(username) {
  var p = new Promise(function (resolve) {
    var isNexted = false;

    try {
      var limiterConfig = _cli.config.users.limiter;

      var client = _redis["default"].createClient();

      client.on('error', function () {
        if (!isNexted) {
          isNexted = true;
          resolve();
        }
      });
      var limit = new _ratelimiter["default"]({
        id: username,
        db: client,
        duration: limiterConfig.duration,
        max: limiterConfig.max
      });
      limit.get(function (err, limit) {
        if (err) {
          resolve();
        } else {
          resolve(limit);
        }
      });
    } catch (e) {
      resolve();
    }
  });
  return p;
}
/**
 * getGravatarImage("bessong@gmail.com", ".jpg?s=200")
 * @param  {[type]} email [description]
 * @param  {[type]} args  [description]
 * @return {[type]}       [description]
 */


function getGravatarImage(email, args) {
  args = args || '';
  var BASE_URL = '//www.gravatar.com/avatar/';
  return (BASE_URL + _cli.coreUtils.text.md5(email) + args).trim();
}
/**
 * getActivity()
 * @param  {[type]} email [description]
 * @param  {[type]} args  [description]
 * @return {[type]}       [description]
 */


function getActivity() {
  var pathToActivity = _path["default"].join(_cli.config.root, _cli.config.users.activity.path);

  if (_cli.config.users.enable && _cli.config.users.activity.active) {
    if (_fsExtra["default"].existsSync(_path["default"].join(pathToActivity, 'activity.json'))) {
      var acArray = _cli.coreUtils.file.getJson(_path["default"].join(pathToActivity, 'activity.json'));

      return acArray;
    }
  }

  return [];
}

function addActivity(activity) {
  var pathToActivity = _path["default"].join(_cli.config.root, _cli.config.users.activity.path);

  var pathToActivityFile = _path["default"].join(pathToActivity, 'activity.json');

  var acArray = getActivity();
  if (acArray.length >= _cli.config.users / activity.history) acArray.shift();
  acArray.push(activity);

  if (_cli.config.users.enable && _cli.config.users.activity.active) {
    _fsExtra["default"].exists(pathToActivityFile, function (exists) {
      if (!exists) {
        _fsExtra["default"].mkdir(_path["default"].join(pathToActivity), function () {
          _fsExtra["default"].writeJson(pathToActivityFile, acArray, function (err) {
            if (err) console.log('save activity error: ', err);
          });
        });
      } else {
        _fsExtra["default"].writeJson(pathToActivityFile, acArray, function (err) {
          if (err) console.log('save activity error: ', err);
        });
      }
    });
  }

  return acArray;
}