#!/usr/bin/env node
"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _operations = require("./cli/cms/operations");

var _plugins = _interopRequireDefault(require("./cli/extend/plugins"));

var _config = _interopRequireDefault(require("./cli/core/config/config"));

var _child_process = require("child_process");

var _path = _interopRequireDefault(require("path"));

var _commander = _interopRequireDefault(require("commander"));

var _package = _interopRequireDefault(require("../package"));

var _cliColor = _interopRequireDefault(require("cli-color"));

var _surge = _interopRequireDefault(require("surge"));

var surge = new _surge["default"]();
var hooks = {};
var deployPlugins = [];
var websiteName = null;
var dir = null;

var deployWebsite = function deployWebsite() {
  var create = new _operations.initSite();
  create.askDeploymentQuestions().then(function (answers) {
    if (answers.deploy) {
      var json = null;

      if (answers.which === "on surge (it's free !)") {
        deployPlugins.push('abecms/abe-deployer-surge');
        json = {
          deployers: {
            surge: {
              active: true,
              domain: answers.domain
            }
          }
        };

        _config["default"].save(json);

        surge.login({
          postAuth: installPlugins
        })([]);
      } else if (answers.which === 'a github repository') {
        deployPlugins.push('abecms/abe-deployer-git');
        json = {
          deployers: {
            git: {
              active: true,
              repository: answers.repository,
              branch: answers.branch,
              username: answers.username,
              email: answers.email
            }
          }
        };

        _config["default"].save(json);

        installPlugins();
      } else if (answers.which === 'a S3 bucket') {
        deployPlugins.push('abecms/abe-deployer-s3');
        json = {
          deployers: {
            s3: {
              active: true,
              region: answers.region,
              accessKeyId: answers.accessKeyId,
              secretAccessKey: answers.secretAccessKey,
              bucket: answers.bucket,
              prefix: answers.prefix
            }
          }
        };

        _config["default"].save(json);

        installPlugins();
      } else if (answers.which === 'via (S)FTP') {
        deployPlugins.push('abecms/abe-deployer-sftp');
        json = {
          deployers: {
            sftp: {
              active: true,
              host: answers.host,
              username: answers.username,
              remoteDir: answers.remoteDir,
              protocol: answers.protocol
            }
          }
        };

        if (answers.requiresType === 'It requires a password') {
          json.deployers.sftp.requiresPassword = true;
          json.deployers.sftp.requireSSHKey = false;
          json.deployers.sftp.password = answers.password;
        } else {
          json.deployers.sftp.requiresPassword = false;
          json.deployers.sftp.requireSSHKey = true;
          json.deployers.sftp.sshKeyPath = answers.sshKeyPath;
        }

        _config["default"].save(json);

        installPlugins();
      }
    } else {
      installPlugins();
    }
  });
};

var installWebsite = function installWebsite() {
  var create = new _operations.initSite();
  create.askQuestions().then(function (answers) {
    dir = _path["default"].join(process.cwd(), answers.name);
    websiteName = answers.name;

    if (process.env.ROOT) {
      dir = _path["default"].join(process.env.ROOT, answers.name);
    }

    _config["default"].root = dir;
    process.env['HOME'] = dir;
    create.init(dir).then(function () {
      return create.updateSecurity(answers);
    }).then(function () {
      return create.updateTheme(answers);
    }).then(function () {
      return deployWebsite();
    });
  });
};

var installPlugins = function installPlugins() {
  var create = new _operations.initSite();
  create.askPluginsQuestions().then(function (answers) {
    if (answers.plugins && answers.plugins.length > 0) {
      answers.plugins.forEach(function (plugin) {
        console.log('installing the plugin: ' + plugin);

        if (typeof plugin !== 'undefined' && plugin !== null) {
          _plugins["default"].instance.install(dir, plugin);
        } else {
          _plugins["default"].instance.install(dir);
        }
      });
    }

    if (deployPlugins && deployPlugins.length > 0) {
      deployPlugins.forEach(function (plugin) {
        console.log("installing the plugin: ".concat(plugin));

        if (typeof plugin !== 'undefined' && plugin !== null) {
          _plugins["default"].instance.install(dir, plugin);
        }
      });
    }

    console.log(_cliColor["default"].green("Yeahhh! Your Abe site ".concat(websiteName, " is ready to launch!  \uD83D\uDE80  \n")), _cliColor["default"].cyan("cd ".concat(websiteName, "\n")), _cliColor["default"].cyan("abe serve -i"));
  });
};

_commander["default"].storeOptionsAsProperties(true);

_commander["default"].version(_package["default"].version).option('-v, --version', 'version'); // Dev: ./node_modules/.bin/babel-node --presets @babel/preset-env src/index.js init


_commander["default"].command('init').alias('i').description('init a new abe website').action(function (options) {
  installWebsite();
}).on('--help', function () {
  console.log('  Examples:\n', '\n', '   $ abe init\n', '');
});

_commander["default"].command('create [path]').alias('c').description('create a new abe project').action(function (dest) {
  dest = dest != null ? dest : '';

  var dir = _path["default"].join(process.cwd(), dest);

  if (process.env.ROOT) {
    dir = _path["default"].join(process.env.ROOT, dest);
  }

  var create = new _operations.initSite();

  if (typeof dir !== 'undefined' && dir !== null && dest !== '') {
    create.init(dir).then(function () {
      console.log(_cliColor["default"].green('Yeahhh! Your Abe site ' + dest + ' is ready to launch!  🚀  '), _cliColor["default"].cyan("\ncd ".concat(dest, " \nabe serve -i")));
    });
  } else {
    console.log('error creating the project');
  }
}).on('--help', function () {
  console.log('  Examples:\n', '\n', '    $ abe create\n', '    $ abe create [destination]\n', '');
});

_commander["default"].command('generate-posts').alias('gp').description('save post to file with type from folder').option('-t, --type [type]', 'posts status draft|other').option('-p, --path [path]', 'path /relative/path').option('-d, --destination [destination]', 'folder to save result').action(function (options) {
  var dir = process.cwd();

  if (process.env.ROOT) {
    dir = process.env.ROOT.replace(/\/$/, '');
  }

  process.env.DEBUG = 'generate-posts:*';
  var generateArgs = ['--harmony', __dirname + '/cli/process/generate-posts.js', 'ABE_WEBSITE=' + dir];

  if (options.destination != null) {
    generateArgs.push('ABE_DESTINATION=' + options.destination);
  }

  if (options.path != null) {
    generateArgs.push('ABE_PATH=' + options.path);
  }

  if (options.type != null) {
    generateArgs.push('ABE_STATUS=' + options.type);
  }

  var generate;

  if (__dirname.indexOf('dist') > -1) {
    generate = (0, _child_process.spawn)('node', generateArgs, {
      shell: true,
      stdio: 'inherit'
    });
  } else {
    generate = (0, _child_process.spawn)(_path["default"].join(__dirname, '..', 'node_modules', '.bin', 'babel'), generateArgs, {
      shell: true,
      stdio: 'inherit'
    });
  }

  generate.on('close', function (code) {
    console.log('child process exited with code ' + code);
    process.exit(0);
  });
}).on('--help', function () {
  console.log('  Examples:\n', '\n', '    $ abe generate-posts --path /test --destination result --status publish\n', '');
});

_commander["default"].command('serve').alias('s').description('create a http server for abe in development mode (debugger + livereload). If you want to deactivate development mode, use -e option').option('-p, --port [number]', 'change port of the web server').option('-i, --interactive', 'open browser on web server startup').option('-t, --templates [path]', 'give an absolute or relative path to your templates').option('-a, --assets [path]', 'give an absolute or relative path to your assets').option('-d, --destination [path]', 'give an absolute or relative path to your destination directory').option('-j, --json [path]', 'give an absolute or relative path to your data directory').option('-e, --env [development|production|...]', 'Abe is launched in development mode by default. Use another value to deactivate development mode. You may also use a global env variable NODE_ENV.').action(function (options) {
  var environment = process.env;
  var dir = process.cwd();
  var command;

  if (environment.ROOT) {
    dir = environment.ROOT;
  }

  environment.ROOT = dir;

  if (typeof environment.NODE_ENV == 'undefined') {
    if (options.env != null && options.env !== 'development' && options.env !== 'dev') {
      environment.NODE_ENV = options.env;
    } else {
      environment.NODE_ENV = 'development';
    }
  }

  if (typeof options.port != 'undefined') {
    environment.PORT = options.port;
  }

  if (typeof options.templates != 'undefined') {
    environment.ABE_TEMPLATES_PATH = options.templates;
  }

  if (typeof options.assets != 'undefined') {
    environment.ABE_ASSETS_PATH = options.assets;
  }

  if (typeof options.destination != 'undefined') {
    environment.ABE_DESTINATION_PATH = options.destination;
  }

  if (typeof options.json != 'undefined') {
    environment.ABE_JSON_PATH = options.json;
  }

  if (__dirname.indexOf('dist') > -1) {
    command = 'node --harmony ./dist/server/index.js';
  } else {
    command = _path["default"].join(__dirname, '..', 'node_modules', '.bin', 'babel') + ' --harmony ./src/server/index.js';
  }

  if (options.interactive != null) {
    command = 'OPENURL=1 ' + command;
  }

  process.chdir(__dirname + '/../');
  console.log('website started : ' + dir);
  var cp = (0, _child_process.exec)(command, {
    env: environment,
    maxBuffer: 1024 * 500
  }, function (err, out, code) {
    try {
      if (err instanceof Error) throw err;
      process.stderr.write(err);
      process.stdout.write(out);
    } catch (e) {}

    process.exit(code);
  });
  cp.stderr.pipe(process.stderr);
  cp.stdout.pipe(process.stdout);
}).on('--help', function () {
  console.log('  Examples:\n', '\n', '    $ abe serve\n', '');
});

_commander["default"].command('install [plugin]').description('install abe plugin(s)').action(function (plugin) {
  var dir = process.cwd();

  if (process.env.ROOT) {
    dir = process.env.ROOT.replace(/\/$/, '');
  }

  if (typeof plugin !== 'undefined' && plugin !== null) {
    _plugins["default"].instance.install(dir, plugin);
  } else {
    _plugins["default"].instance.install(dir);
  }
}).on('--help', function () {
  console.log('  Examples:\n', '\n', '    $ abe install\n', '    $ abe install [plugin]', '');
});

_commander["default"].command('uninstall <plugin>').alias('un').description('uninstall abe plugin(s)').action(function (plugin) {
  var dir = process.env.ROOT.replace(/\/$/, '');

  if (process.env.ROOT) {
    dir = process.env.ROOT.replace(/\/$/, '');
  }

  if (typeof plugin !== 'undefined' && plugin !== null) {
    _plugins["default"].instance.uninstall(dir, plugin);
  }
}).on('--help', function () {
  console.log(['  Examples:', '', '    $ abe uninstall', '    $ abe uninstall [plugin]', '']);
}); // Surge


_commander["default"].command('login').action(surge.login(hooks)).description('Login on Surge to publish projects to the web.');

hooks.preAuth = function (req, next) {
  console.log('');

  if (req.authed) {
    console.log("       Hello ".concat(req.creds.email, " !"));
  } else {
    console.log('       Welcome!');
  }

  console.log('');
  next();
};

_commander["default"].parse(process.argv);