"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _express = _interopRequireDefault(require("express"));

var _https = _interopRequireDefault(require("https"));

var _fs = _interopRequireDefault(require("fs"));

var _expressSession = _interopRequireDefault(require("express-session"));

var _helmet = _interopRequireDefault(require("helmet"));

var _bodyParser = _interopRequireDefault(require("body-parser"));

var _expressHandlebars = _interopRequireDefault(require("express-handlebars"));

var _path = _interopRequireDefault(require("path"));

var _connectBusboy = _interopRequireDefault(require("connect-busboy"));

var _cliColor = _interopRequireDefault(require("cli-color"));

var _openurl = _interopRequireDefault(require("openurl"));

var _connectFlash = _interopRequireDefault(require("connect-flash"));

var _cookieParser = _interopRequireDefault(require("cookie-parser"));

var _csurf = _interopRequireDefault(require("csurf"));

var _passport = _interopRequireDefault(require("passport"));

var _handlebarsLayouts = _interopRequireDefault(require("handlebars-layouts"));

var _expressValidator = require("express-validator");

var _cli = require("../cli");

var _middlewares = require("./middlewares");

var _getMain = _interopRequireDefault(require("./routes/get-main"));

//import getHome from './routes/get-home'
require('events').EventEmitter.defaultMaxListeners = 100;
var abePort = null;
if (_cli.config.port) abePort = _cli.config.port;
if (process.env.PORT) abePort = process.env.PORT;
process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
var partialsDir = [_path["default"].join(__dirname, 'views/partials/'), _path["default"].join(_cli.config.root, _cli.config.custom)];

var pluginsPartialsDir = _cli.abeExtend.plugins.instance.getPartials();

Array.prototype.forEach.call(pluginsPartialsDir, function (pluginPartialsDir) {
  partialsDir.push(pluginPartialsDir);
});

var html = _expressHandlebars["default"].create({
  extname: ".".concat(_cli.config.files.templates.extension),
  helpers: {
    abe: _cli.compileAbe,
    listPage: _cli.listPage,
    math: _cli.math,
    printInput: _cli.printInput,
    abeImport: _cli.abeImport,
    moduloIf: _cli.moduloIf,
    testObj: _cli.testObj,
    notEmpty: _cli.notEmpty,
    printJson: _cli.printJson,
    className: _cli.className,
    attrAbe: _cli.attrAbe,
    folders: _cli.folders,
    printConfig: _cli.printConfig,
    ifIn: _cli.ifIn,
    ifCond: _cli.ifCond,
    isAuthorized: _cli.isAuthorized
  },
  partialsDir: partialsDir
});

var handlebars = html.handlebars; // I register the layout plugin

handlebars.registerHelper((0, _handlebarsLayouts["default"])(handlebars)); // I load the partials in Handlebars for the layout plugin

var partials = _path["default"].join(__dirname, 'views/partials/');

partialsDir.forEach(function (pathDir) {
  _fs["default"].readdirSync(partials).forEach(function (file) {
    var source = _fs["default"].readFileSync(partials + file, "utf8");

    var partial = /(.+)\.html/.exec(file).pop();
    handlebars.registerPartial(partial, source);
  });
});
var opts = {};

if (_cli.coreUtils.file.exist(_path["default"].join(_cli.config.root, 'cert.pem'))) {
  opts = {
    key: _fs["default"].readFileSync(_path["default"].join(_cli.config.root, 'key.pem')),
    cert: _fs["default"].readFileSync(_path["default"].join(_cli.config.root, 'cert.pem'))
  };
}

var app = (0, _express["default"])(opts);
var server; // Instantiate Singleton Manager (which lists all blog files)

_cli.Manager.instance.init();

app.set('config', _cli.config.getConfigByWebsite());
app.use((0, _connectFlash["default"])());
app.use((0, _cookieParser["default"])());
app.use(_bodyParser["default"].urlencoded({
  limit: '1gb',
  extended: true,
  parameterLimit: 50000
})); //app.use(expressValidator())

app.use((0, _csurf["default"])({
  cookie: {
    secure: _cli.config.cookie.secure
  }
}));
app.use(function (req, res, next) {
  if (req.url.indexOf('/abe/') > -1) {
    res.locals._csrf = req.csrfToken();
  }

  next();
});
app.use(_bodyParser["default"].json({
  limit: '1gb'
}));

if (_cli.config.security === true) {
  app.use((0, _helmet["default"])());
  app.use(_helmet["default"].csp({
    directives: {
      defaultSrc: ["'self'"],
      scriptSrc: ["'self'"].concat(_cli.config.csp.scriptSrc),
      styleSrc: ["'self'", "'unsafe-inline'"].concat(_cli.config.csp.styleSrc),
      imgSrc: ["'self'", 'data:'].concat(_cli.config.csp.imgSrc),
      // frameSrc: ["'self'"],
      childSrc: ["'self'"].concat(_cli.config.csp.childSrc),
      frameAncestors: ["'self'"].concat(_cli.config.csp.frameAncestors),
      mediaSrc: ["'self'"].concat(_cli.config.csp.mediaSrc),
      fontSrc: ["'self'"].concat(_cli.config.csp.fontSrc),
      connectSrc: ["'self'"].concat(_cli.config.csp.connectSrc),
      sandbox: ['allow-same-origin', 'allow-scripts', 'allow-modals', 'allow-popups', 'allow-forms'],
      reportUri: '/report-violation',
      objectSrc: [] // An empty array allows nothing through

    },
    reportOnly: false,
    // Set to true if you only want browsers to report errors, not block them
    setAllHeaders: false,
    // Set to true if you want to blindly set all headers: Content-Security-Policy, X-WebKit-CSP, and X-Content-Security-Policy.
    disableAndroid: false,
    // Set to true if you want to disable CSP on Android where it can be buggy.
    browserSniff: true // Set to false if you want to completely disable any user-agent sniffing. This may make the headers less compatible but it will be much faster. This defaults to `true`.

  }));
}

var port = abePort !== null ? abePort : 3000;
port = _cli.abeExtend.hooks.instance.trigger('beforeExpress', port);
app.set('views', _path["default"].join(__dirname, '/views'));
app.engine('.html', html.engine);
app.set('view engine', '.html');
app.locals.layout = false;
app.use(_middlewares.middleCheckCsrf);
app.use(_middlewares.middleIsAuthorized);
app.use(_middlewares.middleLogin);
app.use(_middlewares.middleWebsite);

if (process.env.NODE_ENV === 'development') {
  _cli.Manager.instance.initDev();

  process.on('uncaughtException', function (err) {
    // We need to trap this error which is sent on websocket client connection
    if (err.code !== 'ECONNRESET') {
      throw err;
    }
  });
  app.use(_middlewares.middleLiveReload);
}

app.use(_express["default"]["static"](__dirname + '/public'));
app.use(function (req, res, next) {
  res.header('Access-Control-Allow-Origin', '*');
  res.header('Access-Control-Allow-Headers', 'Origin, X-Access-Token, X-Requested-With, Content-Type, Accept');
  next();
});
var publish = _cli.Manager.instance.pathPublish;
app.use(_express["default"]["static"](publish));

if (_cli.coreUtils.file.exist(_cli.Manager.instance.pathPartials)) {
  app.use(_express["default"]["static"](_cli.Manager.instance.pathPartials));
}

if (_cli.config.custom !== '') {
  if (_cli.coreUtils.file.exist(_path["default"].join(_cli.config.root, _cli.config.custom))) {
    app.use(_express["default"]["static"](_path["default"].join(_cli.config.root, _cli.config.custom)));
  }
}

var pluginsPartials = _cli.abeExtend.plugins.instance.getPartials();

Array.prototype.forEach.call(pluginsPartials, function (pluginPartials) {
  app.use(_express["default"]["static"](pluginPartials));
});
app.use(_express["default"]["static"](__dirname + '/node_modules/handlebars/dist'));
app.use((0, _connectBusboy["default"])({
  limits: {
    fileSize: _cli.config.upload.fileSizelimit
  }
}));
app.use(_bodyParser["default"].json());
app.use(_bodyParser["default"].urlencoded({
  extended: true
})); // depending on the way you serve this app, cookie.secure will be set
// in Production, this app has to be reverse-proxified

app.use((0, _expressSession["default"])({
  name: 'sessionId',
  secret: _cli.config.sessionSecret,
  resave: false,
  saveUninitialized: true,
  cookie: {
    secure: _cli.config.cookie.secure
  },
  proxy: true
}));
app.use(_passport["default"].initialize());
app.use(_passport["default"].session());

_cli.abeExtend.hooks.instance.trigger('afterExpress', app, _express["default"]); // if served through pm2 with sockets


if (_fs["default"].existsSync(port) && _fs["default"].lstatSync(port).isSocket()) {
  _fs["default"].unlink(port);
}

if (_cli.coreUtils.file.exist(_path["default"].join(_cli.config.root, 'cert.pem'))) {
  server = _https["default"].createServer(opts, app);
  server.listen(port, function () {
    console.log(_cliColor["default"].green("\nserver running at https://localhost:".concat(port, "/")));
    if (process.env.OPENURL) _openurl["default"].open("https://localhost:".concat(port, "/abe/"));
  });
} else {
  server = app.listen(port, function () {
    console.log(_cliColor["default"].green("\nserver running at http://localhost:".concat(port, "/")));
    if (process.env.OPENURL) _openurl["default"].open("http://localhost:".concat(port, "/abe/"));
  });
}

if (_cli.config.websocket.active === true) {
  var io = require('socket.io')(server);

  app.use(function (request, response, next) {
    request.io = io;
    next();
  });
}

server.on('error', onError);

function onError(error) {
  if (error.syscall !== 'listen') throw error;
  var bind = typeof port === 'string' ? 'Pipe ' + port : 'Port ' + port;

  switch (error.code) {
    case 'EACCES':
      console.error(bind + ' requires elevated privileges');
      process.exit(1);
      break;

    case 'EADDRINUSE':
      console.error(bind + ' is already in use');
      process.exit(1);
      break;

    default:
      throw error;
  }
}

var cleanup = function cleanup() {
  server.close(function () {
    console.log('Closed out remaining connections.');
    process.exit();
  });
  setTimeout(function () {
    console.log('Could not close connections in time, forcing shut down');
    process.exit(1);
  }, 10 * 1000);
};

process.on('SIGINT', cleanup);
process.on('SIGTERM', cleanup); // important : require here so config.root is defined

var routes = require('./routes');

app.use(routes["default"]); // This static path is mandatory for relative path to statics in templates

app.use('/abe/editor', _express["default"]["static"](publish));
app.use('/abe/page', _express["default"]["static"](publish));
app.get('/abe*', _getMain["default"]);