"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _package = _interopRequireDefault(require("../../../package"));

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _handlebars = _interopRequireDefault(require("handlebars"));

var _cli = require("../../cli");

/**
 * This route returns all the available routes in HTML format
 * @param  {[type]}   router [description]
 * @param  {[type]}   req    [description]
 * @param  {[type]}   res    [description]
 * @param  {Function} next   [description]
 * @return {[type]}          [description]
 */
var route = function route(router, req, res, next) {
  _cli.abeExtend.hooks.instance.trigger('beforeRoute', req, res, next);

  var routes = router.stack;
  var urls = [];
  var html = '';
  urls.push({
    url: '/abe',
    method: '/GET',
    regex: '/abe$'
  });
  Array.prototype.forEach.call(routes, function (route) {
    urls.push({
      url: route.route.path,
      method: Object.keys(route.route.methods)[0].toUpperCase(),
      regex: route.route.path.replace(/\*$/, '') + '.*'
    });
  });

  var page = _path["default"].join(__dirname + '/../views/list-url.html');

  if (_cli.coreUtils.file.exist(page)) {
    html = _fs["default"].readFileSync(page, 'utf8');
  }

  var workflowUrl = {};
  Array.prototype.forEach.call(_cli.config.users.workflow, function (flow) {
    workflowUrl[flow] = [{
      url: "/abe/operations/edit/".concat(flow),
      action: 'edit',
      workflow: flow
    }, {
      url: "/abe/operations/delete/".concat(flow),
      action: 'delete',
      workflow: flow
    }];
  });

  var template = _handlebars["default"].compile(html, {
    noEscape: true
  });

  var tmp = template({
    urls: urls,
    user: res.user,
    config: _cli.config,
    roles: _cli.config.users.roles,
    workflow: _cli.config.users.workflow,
    workflowUrl: workflowUrl,
    isRoles: true,
    manager: {
      config: JSON.stringify(_cli.config)
    },
    abeVersion: _package["default"].version
  });
  res.cookie('csrf-token', res.locals.csrfToken);
  return res.send(tmp);
};

var _default = route;
exports["default"] = _default;