"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _package = _interopRequireDefault(require("../../../package"));

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _handlebars = _interopRequireDefault(require("handlebars"));

var _cli = require("../../cli");

/**
 * this route returns the workflow list in HTML format
 * @param  {[type]}   router [description]
 * @param  {[type]}   req    [description]
 * @param  {[type]}   res    [description]
 * @param  {Function} next   [description]
 * @return {[type]}          [description]
 */
var route = function route(router, req, res, next) {
  _cli.abeExtend.hooks.instance.trigger('beforeRoute', req, res, next);

  var routes = router.stack;
  var urls = [];
  var html = '';
  Array.prototype.forEach.call(routes, function (route) {
    urls.push({
      url: route.route.path,
      method: Object.keys(route.route.methods)[0].toUpperCase(),
      regex: route.route.path.replace(/\*$/, '') + '.*'
    });
  });

  var page = _path["default"].join(__dirname + '/../views/list-workflow.html');

  if (_cli.coreUtils.file.exist(page)) {
    html = _fs["default"].readFileSync(page, 'utf8');
  }

  var workflowUrl = {};
  var previous = '';
  var nextWorkflow = '';
  Array.prototype.forEach.call(_cli.config.users.workflow, function (flow) {
    var current = false;

    if (flow != 'publish') {
      Array.prototype.forEach.call(_cli.config.users.workflow, function (flowCheck) {
        if (current) {
          nextWorkflow = flowCheck;
          current = false;
        }

        if (flow === flowCheck) {
          current = true;
        }
      });
    } else {
      nextWorkflow = 'draft';
    }

    workflowUrl[flow] = [{
      url: "/abe/operations/edit/".concat(flow),
      action: 'edit',
      workflow: flow,
      previous: previous,
      next: nextWorkflow
    }, {
      url: "/abe/operations/delete/".concat(flow),
      action: 'delete',
      workflow: flow,
      previous: previous,
      next: nextWorkflow
    }, {
      url: "/abe/operations/submit/".concat(flow),
      action: 'submit',
      workflow: flow,
      previous: previous,
      next: nextWorkflow
    }];

    if (flow !== 'draft') {
      workflowUrl[flow].push({
        url: "/abe/operations/reject/".concat(flow),
        action: 'reject',
        workflow: flow,
        previous: previous,
        next: nextWorkflow
      });
    }

    previous = flow;
  });

  var template = _handlebars["default"].compile(html, {
    noEscape: true
  });

  var tmp = template({
    urls: urls,
    user: res.user,
    config: _cli.config,
    roles: _cli.config.users.roles,
    workflow: _cli.config.users.workflow,
    isWorkflow: 1,
    workflowUrl: workflowUrl,
    manager: {
      config: JSON.stringify(_cli.config)
    },
    abeVersion: _package["default"].version
  });
  res.cookie('csrf-token', res.locals.csrfToken);
  return res.send(tmp);
};

var _default = route;
exports["default"] = _default;