"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _cookies = _interopRequireDefault(require("cookies"));

var _moment = _interopRequireDefault(require("moment"));

var _passportLocal = _interopRequireDefault(require("passport-local"));

var _passport = _interopRequireDefault(require("passport"));

var _jwtSimple = _interopRequireDefault(require("jwt-simple"));

var _cli = require("../../../cli");

/**
 * Strategy
 */
_passport["default"].use(new _passportLocal["default"](function (username, password, done) {
  _cli.User.utils.findByUsername(username, function (err, user) {
    if (err) {
      return done(err);
    }

    if (!user) {
      return done(null, false, {
        message: 'Incorrect username or password.'
      });
    }

    if (!_cli.User.utils.isValid(user, password)) {
      return done(null, false, {
        message: 'Incorrect username or password.'
      });
    }

    return done(null, user);
  });
}));

_passport["default"].serializeUser(function (user, done) {
  done(null, user.id);
});

_passport["default"].deserializeUser(function (id, done) {
  _cli.User.utils.find(id, function (err, user) {
    done(err, user);
  });
});

var route = function route(req, res, next) {
  _cli.User.utils.loginLimitTry(req.body.username).then(function (limit) {
    if (limit != null) {
      // all good
      if (!limit.remaining) {
        res.status(401);
        res.json({
          status: 401,
          message: 'Invalid credentials. Rate limit exceeded'
        });
        return;
      }
    }

    _passport["default"].authenticate('local', {
      session: false
    }, function (err, user, info) {
      var secret = _cli.config.users.secret;

      if (err) {
        return next(err);
      }

      if (!user) {
        res.status(401);
        res.json({
          status: 401,
          message: 'Invalid credentials'
        });
        return;
      }

      var expires = (0, _moment["default"])().add(7, 'days').valueOf();

      var token = _jwtSimple["default"].encode({
        iss: user.id,
        exp: expires,
        username: user.username,
        name: user.name,
        email: user.email,
        role: user.role
      }, secret);

      var cookies = new _cookies["default"](req, res, {
        secure: _cli.config.cookie.secure
      });
      cookies.set('x-access-token', token);
      var result = {
        id_token: token
      };
      res.set('Content-Type', 'application/json');
      res.send(JSON.stringify(result));
    })(req, res, next);
  });
};

var _default = route;
exports["default"] = _default;