"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _crypto = _interopRequireDefault(require("crypto"));

var _nodemailer = _interopRequireDefault(require("nodemailer"));

var _path = _interopRequireDefault(require("path"));

var _cli = require("../../../../cli");

function showHtml(res, req, info) {
  var resHtml = '';

  var page = _path["default"].join(__dirname + '/../../../views/users/forgot.html');

  if (_cli.coreUtils.file.exist(page)) {
    resHtml = _fsExtra["default"].readFileSync(page, 'utf8');
  }

  var template = _cli.Handlebars.compile(resHtml, {
    noEscape: true
  });

  var tmp = template({
    csrfToken: res.locals.csrfToken,
    config: JSON.stringify(_cli.config),
    express: {
      req: req,
      res: res
    },
    token: req.query.token,
    info: info
  });
  return res.send(tmp);
}

var route = function route(req, res) {
  var html;

  if (typeof req.query.email !== 'undefined' && req.query.email !== null) {
    _cli.User.utils.findByEmail(req.query.email, function (err, user) {
      if (err) {
        return res.status(200).json({
          success: 1
        });
      }

      if (!user) {
        return showHtml(res, req, 'Email not found');
      }

      _crypto["default"].randomBytes(20, function (err, buf) {
        var resetPasswordToken = buf.toString('hex');
        var forgotExpire = _cli.config.forgotExpire;

        _cli.User.operations.update({
          id: user.id,
          resetPasswordToken: resetPasswordToken,
          resetPasswordExpires: Date.now() + forgotExpire * 60 * 1000
        });

        var requestedUrl = req.protocol + '://' + req.get('Host') + '/abe/users/reset?token=' + resetPasswordToken;
        var emailConf = _cli.config.users.email;
        html = emailConf.html || '';

        if (typeof emailConf.templateHtml !== 'undefined' && emailConf.templateHtml !== null) {
          var fileHtml = _path["default"].join(_cli.config.root, emailConf.templateHtml);

          if (_cli.coreUtils.file.exist(fileHtml)) {
            html = _fsExtra["default"].readFileSync(fileHtml, 'utf8');
          }
        }

        var template = _cli.Handlebars.compile(html, {
          noEscape: true
        });

        html = template({
          express: {
            req: req,
            res: res
          },
          forgotUrl: requestedUrl,
          siteUrl: req.protocol + '://' + req.get('Host'),
          user: user
        });
        var from = emailConf.from;
        var to = user.email;
        var subject = emailConf.subject;
        var text = emailConf.text.replace(/\{\{forgotUrl\}\}/g, requestedUrl);
        var html = html.replace(/\{\{forgotUrl\}\}/g, requestedUrl);

        _cli.coreUtils.mail.send(from, to, subject, text, html);

        showHtml(res, req, 'Check your inbox');
      });
    });
  } else {
    showHtml(res, req, req.flash('info'));
  }
};

var _default = route;
exports["default"] = _default;